# -*- coding: utf-8 -*-

from typing import List

import pandas as pd
import requests

from zvtm.contract.api import df_to_db
from zvtm.contract.recorder import TimestampsDataRecorder
from zvtm.domain import Index, IndexStock
from zvtm.recorders.exchange.api import cs_index_stock_api, cn_index_stock_api
from zvtm.utils.time_utils import pre_month_start_date
from zvtm.utils.time_utils import to_pd_timestamp


class ExchangeIndexStockRecorder(TimestampsDataRecorder):
    entity_provider = 'exchange'
    entity_schema = Index

    provider = 'exchange'
    data_schema = IndexStock

    def __init__(self,
                 force_update=False,
                 sleeping_time=5,
                 exchanges=None,
                 entity_ids=None,
                 code=None,
                 codes=None,
                 day_data=False,
                 entity_filters=None,
                 ignore_failed=True,
                 real_time=False,
                 fix_duplicate_way='add',
                 start_timestamp=None,
                 end_timestamp=None,
                 record_history=False) -> None:
        super().__init__(force_update, sleeping_time, exchanges, entity_ids, code, codes, day_data, entity_filters,
                         ignore_failed, real_time, fix_duplicate_way, start_timestamp, end_timestamp)
        self.record_history = record_history

    def init_timestamps(self, entity_item) -> List[pd.Timestamp]:
        last_valid_date = pre_month_start_date()
        if self.record_history:
            # 每个月记录一次
            return [to_pd_timestamp(item) for item in
                    pd.date_range(entity_item.list_date, last_valid_date, freq='M')]
        else:
            return [last_valid_date]

    def record(self, entity, start, end, size, timestamps):
        if entity.publisher == 'cnindex':
            for timestamp in timestamps:
                df = cn_index_stock_api.get_cn_index_stock(code=entity.code, timestamp=timestamp, name=entity.name)
                df_to_db(data_schema=self.data_schema, df=df, provider=self.provider,
                         force_update=True)
        elif entity.publisher == 'csindex':
            # cs index not support history data
            df = cs_index_stock_api.get_cs_index_stock(code=entity.code, timestamp=None, name=entity.name)
            df_to_db(data_schema=self.data_schema, df=df, provider=self.provider,
                     force_update=True)


if __name__ == '__main__':
    # ExchangeIndexMetaRecorder().run()
    ExchangeIndexStockRecorder(codes=['399370']).run()
# the __all__ is generated
__all__ = ['ExchangeIndexStockRecorder']


# -*- coding: utf-8 -*-
# this file is generated by gen_kdata_schema function, dont't change it
from sqlalchemy.orm import declarative_base

from zvtm.contract.register import register_schema
from zvtm.domain.quotes import StockKdataCommon

KdataBase = declarative_base()


class Stock15mKdata(KdataBase, StockKdataCommon):
    __tablename__ = 'stock_15m_kdata'


register_schema(providers=['joinquant', 'em'], db_name='stock_15m_kdata', schema_base=KdataBase, entity_type='stock')

# the __all__ is generated
__all__ = ['Stock15mKdata']


# -*- coding: utf-8 -*-
# the __all__ is generated
__all__ = []

# __init__.py structure:
# common code of the package
# export interface in __all__ which contains __all__ of its sub modules

# import all from submodule stock_1d_ma_stats_factor
from .stock_1d_ma_stats_factor import *
from .stock_1d_ma_stats_factor import __all__ as _stock_1d_ma_stats_factor_all
__all__ += _stock_1d_ma_stats_factor_all

# import all from submodule stock_1d_ma_factor
from .stock_1d_ma_factor import *
from .stock_1d_ma_factor import __all__ as _stock_1d_ma_factor_all
__all__ += _stock_1d_ma_factor_all

# import all from submodule common
from .common import *
from .common import __all__ as _common_all
__all__ += _common_all


# the __all__ is generated
__all__ = []

# __init__.py structure:
# common code of the package
# export interface in __all__ which contains __all__ of its sub modules

# import all from submodule meta
from .meta import *
from .meta import __all__ as _meta_all
__all__ += _meta_all

# import all from submodule holder
from .holder import *
from .holder import __all__ as _holder_all
__all__ += _holder_all

# import all from submodule common
from .common import *
from .common import __all__ as _common_all
__all__ += _common_all

# import all from submodule finance
from .finance import *
from .finance import __all__ as _finance_all
__all__ += _finance_all

# import all from submodule dividend_financing
from .dividend_financing import *
from .dividend_financing import __all__ as _dividend_financing_all
__all__ += _dividend_financing_all

# import all from submodule trading
from .trading import *
from .trading import __all__ as _trading_all
__all__ += _trading_all


# -*- coding: utf-8 -*-
import argparse
from typing import List, Union, Type, Optional

import pandas as pd

from zvtm.contract import IntervalLevel, TradableEntity, AdjustType
from zvtm.contract.api import get_entities, get_schema_by_name
from zvtm.contract.factor import Accumulator
from zvtm.contract.factor import Transformer
from zvtm.domain import Stock
from zvtm.factors.algorithm import MaTransformer, MaAndVolumeTransformer
from zvtm.factors.technical_factor import TechnicalFactor
from zvtm.utils.time_utils import now_pd_timestamp


def get_ma_factor_schema(entity_type: str,
                         level: Union[IntervalLevel, str] = IntervalLevel.LEVEL_1DAY):
    if type(level) == str:
        level = IntervalLevel(level)

    schema_str = '{}{}MaFactor'.format(entity_type.capitalize(), level.value.capitalize())

    return get_schema_by_name(schema_str)


class MaFactor(TechnicalFactor):
    def __init__(self,
                 entity_schema: Type[TradableEntity] = Stock,
                 provider: str = None,
                 entity_provider: str = None,
                 entity_ids: List[str] = None,
                 exchanges: List[str] = None,
                 codes: List[str] = None,
                 start_timestamp: Union[str, pd.Timestamp] = None,
                 end_timestamp: Union[str, pd.Timestamp] = None,
                 columns: List = None,
                 filters: List = None,
                 order: object = None,
                 limit: int = None,
                 level: Union[str, IntervalLevel] = IntervalLevel.LEVEL_1DAY,
                 category_field: str = 'entity_id',
                 time_field: str = 'timestamp',
                 computing_window: int = None,
                 keep_all_timestamp: bool = False,
                 fill_method: str = 'ffill',
                 effective_number: int = None,
                 need_persist: bool = False,
                 only_compute_factor: bool = False,
                 factor_name: str = None,
                 clear_state: bool = False,
                 only_load_factor: bool = False,
                 adjust_type: Union[AdjustType, str] = None,
                 windows=None) -> None:
        if need_persist:
            self.factor_schema = get_ma_factor_schema(entity_type=entity_schema.__name__, level=level)

        if not windows:
            windows = [5, 10, 34, 55, 89, 144, 120, 250]
        self.windows = windows
        transformer: Transformer = MaTransformer(windows=windows)

        super().__init__(entity_schema, provider, entity_provider, entity_ids, exchanges, codes, start_timestamp,
                         end_timestamp, columns, filters, order, limit, level, category_field, time_field,
                         computing_window, keep_all_timestamp, fill_method, effective_number, transformer, None,
                         need_persist, only_compute_factor, factor_name, clear_state, only_load_factor, adjust_type)

    def drawer_factor_df_list(self) -> Optional[List[pd.DataFrame]]:
        return [self.factor_df[self.transformer.indicators]]


class CrossMaFactor(MaFactor):
    def compute_result(self):
        super().compute_result()
        cols = [f'ma{window}' for window in self.windows]
        s = self.factor_df[cols[0]] > self.factor_df[cols[1]]
        current_col = cols[1]
        for col in cols[2:]:
            s = s & (self.factor_df[current_col] > self.factor_df[col])
            current_col = col

        print(self.factor_df[s])
        self.result_df = s.to_frame(name='score')


class VolumeUpMaFactor(TechnicalFactor):

    def __init__(self,
                 entity_schema: Type[TradableEntity] = Stock,
                 provider: str = None,
                 entity_provider: str = None,
                 entity_ids: List[str] = None,
                 exchanges: List[str] = None,
                 codes: List[str] = None,
                 start_timestamp: Union[str, pd.Timestamp] = None,
                 end_timestamp: Union[str, pd.Timestamp] = None,
                 filters: List = None,
                 order: object = None,
                 limit: int = None,
                 level: Union[str, IntervalLevel] = IntervalLevel.LEVEL_1DAY,
                 category_field: str = 'entity_id',
                 time_field: str = 'timestamp',
                 computing_window: int = None,
                 keep_all_timestamp: bool = False,
                 fill_method: str = 'ffill',
                 effective_number: int = None,
                 accumulator: Accumulator = None,
                 need_persist: bool = False,
                 only_compute_factor: bool = False,
                 factor_name: str = None,
                 clear_state: bool = False,
                 only_load_factor: bool = False,
                 adjust_type: Union[AdjustType, str] = None,
                 windows=None,
                 vol_windows=None,
                 turnover_threshold=300000000,
                 over_mode='and') -> None:
        if not windows:
            windows = [250]
        if not vol_windows:
            vol_windows = [30]

        self.windows = windows
        self.vol_windows = vol_windows
        self.turnover_threshold = turnover_threshold
        self.over_mode = over_mode

        columns: List = ['id', 'entity_id', 'timestamp', 'level', 'open', 'close', 'high', 'low', 'volume',
                         'turnover']

        transformer: Transformer = MaAndVolumeTransformer(windows=windows, vol_windows=vol_windows)

        super().__init__(entity_schema, provider, entity_provider, entity_ids, exchanges, codes, start_timestamp,
                         end_timestamp, columns, filters, order, limit, level, category_field, time_field,
                         computing_window, keep_all_timestamp, fill_method, effective_number, transformer, accumulator,
                         need_persist, only_compute_factor, factor_name, clear_state, only_load_factor, adjust_type)

    def compute_result(self):
        super().compute_result()

        # 价格刚上均线
        cols = [f'ma{window}' for window in self.windows]
        filter_se = (self.factor_df['close'] > self.factor_df[cols[0]]) & (
                self.factor_df['close'] < 1.1 * self.factor_df[cols[0]])
        for col in cols[1:]:
            if self.over_mode == 'and':
                filter_se = filter_se & (self.factor_df['close'] > self.factor_df[col])
            else:
                filter_se = filter_se | (self.factor_df['close'] > self.factor_df[col])
        # 放量
        if self.vol_windows:
            vol_cols = [f'vol_ma{window}' for window in self.vol_windows]
            filter_se = filter_se & (self.factor_df['volume'] > 2 * self.factor_df[vol_cols[0]])
            for col in vol_cols[1:]:
                filter_se = filter_se & (self.factor_df['volume'] > 2 * self.factor_df[col])

        # 成交额过滤
        filter_se = filter_se & (self.factor_df['turnover'] > self.turnover_threshold)

        print(self.factor_df[filter_se])
        self.result_df = filter_se.to_frame(name='score')


class CrossMaVolumeFactor(VolumeUpMaFactor):
    def compute_result(self):
        # 均线多头排列
        cols = [f'ma{window}' for window in self.windows]
        filter_se = self.factor_df[cols[0]] > self.factor_df[cols[1]]
        current_col = cols[1]
        for col in cols[2:]:
            filter_se = filter_se & (self.factor_df[current_col] > self.factor_df[col])
            current_col = col

        filter_se = filter_se & (self.factor_df['turnover'] > self.turnover_threshold)

        print(self.factor_df[filter_se])
        self.result_df = filter_se.to_frame(name='score')


if __name__ == '__main__':
    print('start')
    parser = argparse.ArgumentParser()
    parser.add_argument('--level', help='trading level', default='1d',
                        choices=[item.value for item in IntervalLevel])
    parser.add_argument('--start', help='start code', default='000001')
    parser.add_argument('--end', help='end code', default='000005')

    args = parser.parse_args()

    level = IntervalLevel(args.level)
    start = args.start
    end = args.end

    entities = get_entities(provider='eastmoney', entity_type='stock', columns=[Stock.entity_id, Stock.code],
                            filters=[Stock.code >= start, Stock.code < end])

    codes = entities.index.to_list()

    factor = VolumeUpMaFactor(entity_ids=['stock_sz_000338'], start_timestamp='2020-01-01',
                              end_timestamp=now_pd_timestamp(), level=level, need_persist=False)
    print(factor.result_df)
# the __all__ is generated
__all__ = ['get_ma_factor_schema', 'MaFactor', 'CrossMaFactor', 'VolumeUpMaFactor', 'CrossMaVolumeFactor']



# -*- coding: utf-8 -*-
from sqlalchemy import Column, String
from sqlalchemy.orm import declarative_base

from zvtm.contract import Mixin
from zvtm.contract.register import register_schema

StockTagsBase = declarative_base()


class StockTags(StockTagsBase, Mixin):
    __tablename__ = 'stock_tags'

    actor_tag = Column(String(length=64))
    style_tag = Column(String(length=64))
    cycle_tag = Column(String(length=64))
    market_value_tag = Column(String(length=64))


register_schema(providers=['zvt'], db_name='stock_tags',
                schema_base=StockTagsBase)



# -*- coding: utf-8 -*-

from typing import Union

import numpy as np
import pandas as pd

from zvtm.contract import IntervalLevel, AdjustType, Mixin
from zvtm.contract.api import decode_entity_id, get_schema_by_name
from zvtm.utils.pd_utils import pd_is_not_null
from zvtm.utils.time_utils import to_time_str, TIME_FORMAT_DAY, TIME_FORMAT_ISO8601


def get_kdata_schema(entity_type: str,
                     level: Union[IntervalLevel, str] = IntervalLevel.LEVEL_1DAY,
                     adjust_type: Union[AdjustType, str] = None) -> Mixin:
    if type(level) == str:
        level = IntervalLevel(level)
    if type(adjust_type) == str:
        adjust_type = AdjustType(adjust_type)

    # kdata schema rule
    # name:{entity_type.capitalize()}{IntervalLevel.value.capitalize()}Kdata
    if adjust_type and (adjust_type != AdjustType.qfq):
        schema_str = '{}{}{}Kdata'.format(entity_type.capitalize(), level.value.capitalize(),
                                          adjust_type.value.capitalize())
    else:
        schema_str = '{}{}Kdata'.format(entity_type.capitalize(), level.value.capitalize())
    return get_schema_by_name(schema_str)


def get_kdata(entity_id=None, entity_ids=None, level=IntervalLevel.LEVEL_1DAY.value, provider=None, columns=None,
              return_type='df', start_timestamp=None, end_timestamp=None, filters=None, session=None, order=None,
              limit=None, index='timestamp', adjust_type: AdjustType = None):
    assert not entity_id or not entity_ids
    if entity_ids:
        entity_id = entity_ids[0]
    else:
        entity_ids = [entity_id]

    entity_type, exchange, code = decode_entity_id(entity_id)
    data_schema: Mixin = get_kdata_schema(entity_type, level=level, adjust_type=adjust_type)

    return data_schema.query_data(entity_ids=entity_ids, level=level, provider=provider,
                                  columns=columns, return_type=return_type, start_timestamp=start_timestamp,
                                  end_timestamp=end_timestamp, filters=filters, session=session, order=order,
                                  limit=limit,
                                  index=index)


def generate_kdata_id(entity_id, timestamp, level):
    if level >= IntervalLevel.LEVEL_1DAY:
        return "{}_{}".format(entity_id, to_time_str(timestamp, fmt=TIME_FORMAT_DAY))
    else:
        return "{}_{}".format(entity_id, to_time_str(timestamp, fmt=TIME_FORMAT_ISO8601))


def to_high_level_kdata(kdata_df: pd.DataFrame, to_level: IntervalLevel):
    def to_close(s):
        if pd_is_not_null(s):
            return s[-1]

    def to_open(s):
        if pd_is_not_null(s):
            return s[0]

    def to_high(s):
        return np.max(s)

    def to_low(s):
        return np.min(s)

    def to_sum(s):
        return np.sum(s)

    original_level = kdata_df['level'][0]
    entity_id = kdata_df['entity_id'][0]
    provider = kdata_df['provider'][0]
    name = kdata_df['name'][0]
    code = kdata_df['code'][0]

    entity_type, _, _ = decode_entity_id(entity_id=entity_id)

    assert IntervalLevel(original_level) <= IntervalLevel.LEVEL_1DAY
    assert IntervalLevel(original_level) < IntervalLevel(to_level)

    df: pd.DataFrame = None
    if to_level == IntervalLevel.LEVEL_1WEEK:
        # loffset='-2'　用周五作为时间标签
        if entity_type == 'stock':
            df = kdata_df.resample('W', loffset=pd.DateOffset(days=-2)).apply({'close': to_close,
                                                                               'open': to_open,
                                                                               'high': to_high,
                                                                               'low': to_low,
                                                                               'volume': to_sum,
                                                                               'turnover': to_sum})
        else:
            df = kdata_df.resample('W', loffset=pd.DateOffset(days=-2)).apply({'close': to_close,
                                                                               'open': to_open,
                                                                               'high': to_high,
                                                                               'low': to_low,
                                                                               'volume': to_sum,
                                                                               'turnover': to_sum})
    df = df.dropna()
    # id        entity_id  timestamp   provider    code  name level
    df['entity_id'] = entity_id
    df['provider'] = provider
    df['code'] = code
    df['name'] = name

    return df
# the __all__ is generated
__all__ = ['get_kdata_schema', 'get_kdata', 'generate_kdata_id', 'to_high_level_kdata']


# -*- coding: utf-8 -*-
import time

import requests

from zvtm.contract.recorder import FixedCycleDataRecorder
from zvtm.domain import StockMoneyFlow, Stock, StockTradeDay
from zvtm.utils.time_utils import to_pd_timestamp, is_same_date, now_pd_timestamp
from zvtm.utils.utils import to_float


class SinaStockMoneyFlowRecorder(FixedCycleDataRecorder):
    entity_provider = 'joinquant'
    entity_schema = Stock

    provider = 'sina'
    data_schema = StockMoneyFlow

    url = 'http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/MoneyFlow.ssl_qsfx_lscjfb?page=1&num={}&sort=opendate&asc=0&daima={}'

    def init_entities(self):
        super().init_entities()
        # 过滤掉退市的
        self.entities = [entity for entity in self.entities if
                         (entity.end_date is None) or (entity.end_date > now_pd_timestamp())]

    # TODO:more general for the case using StockTradeDay
    def evaluate_start_end_size_timestamps(self, entity):
        start, end, size, timestamps = super().evaluate_start_end_size_timestamps(entity)
        if start:
            trade_day = StockTradeDay.query_data(limit=1, order=StockTradeDay.timestamp.desc(), return_type='domain')
            if trade_day:
                if is_same_date(trade_day[0].timestamp, start):
                    size = 0
        return start, end, size, timestamps

    def generate_url(self, code, number):
        return self.url.format(number, code)

    def get_data_map(self):
        return {}

    def record(self, entity, start, end, size, timestamps):
        param = {
            'url': self.generate_url(code='{}{}'.format(entity.exchange, entity.code), number=size),
            'security_item': entity
        }

        resp = requests.get(param['url'])
        # {opendate:"2019-04-29",trade:"10.8700",changeratio:"-0.0431338",turnover:"74.924",netamount:"-2903349.8500",
        # ratioamount:"-0.155177",r0:"0.0000",r1:"2064153.0000",r2:"6485031.0000",r3:"10622169.2100",r0_net:"0.0000",
        # r1_net:"2064153.0000",r2_net:"-1463770.0000",r3_net:"-3503732.8500"}
        opendate = "opendate"
        trade = "trade"
        changeratio = 'changeratio'
        turnover = 'turnover'
        netamount = 'netamount'
        ratioamount = 'ratioamount'
        r0 = 'r0'
        r1 = 'r1'
        r2 = 'r2'
        r3 = 'r3'
        r0_net = 'r0_net'
        r1_net = 'r1_net'
        r2_net = 'r2_net'
        r3_net = 'r3_net'

        json_list = []

        try:
            json_list = eval(resp.text)
        except Exception as e:
            resp.encoding = 'GBK'
            self.logger.error(resp.text)
            time.sleep(60 * 5)

        result_list = []
        for item in json_list:
            amount = to_float(item['r0']) + to_float(item['r1']) + to_float(item['r2']) + to_float(item['r3'])

            result = {
                'timestamp': to_pd_timestamp(item['opendate']),
                'name': entity.name,
                'close': to_float(item['trade']),
                'change_pct': to_float(item['changeratio']),
                'turnover_rate': to_float(item['turnover']) / 10000,
                'net_inflows': to_float(item['netamount']),
                'net_inflow_rate': to_float(item['ratioamount']),
                #     # 主力=超大单+大单
                #     net_main_inflows = Column(Float)
                #     net_main_inflow_rate = Column(Float)
                #     # 超大单
                #     net_huge_inflows = Column(Float)
                #     net_huge_inflow_rate = Column(Float)
                #     # 大单
                #     net_big_inflows = Column(Float)
                #     net_big_inflow_rate = Column(Float)
                #
                #     # 中单
                #     net_medium_inflows = Column(Float)
                #     net_medium_inflow_rate = Column(Float)
                #     # 小单
                #     net_small_inflows = Column(Float)
                #     net_small_inflow_rate = Column(Float)
                'net_main_inflows': to_float(item['r0_net']) + to_float(item['r1_net']),

                'net_huge_inflows': to_float(item['r0_net']),

                'net_big_inflows': to_float(item['r1_net']),

                'net_medium_inflows': to_float(item['r2_net']),

                'net_small_inflows': to_float(item['r3_net']),
            }

            if amount != 0:
                result['net_main_inflow_rate'] = (to_float(item['r0_net']) + to_float(item['r1_net'])) / amount
                result['net_huge_inflow_rate'] = to_float(item['r0_net']) / amount
                result['net_big_inflow_rate'] = to_float(item['r1_net']) / amount
                result['net_medium_inflow_rate'] = to_float(item['r2_net']) / amount
                result['net_small_inflow_rate'] = to_float(item['r3_net']) / amount

            result_list.append(result)

        return result_list



if __name__ == '__main__':
    SinaStockMoneyFlowRecorder(codes=['000406']).run()
    # SinaStockMoneyFlowRecorder().run()
# the __all__ is generated
__all__ = ['SinaStockMoneyFlowRecorder']


# -*- coding: utf-8 -*-
# this file is generated by gen_kdata_schema function, dont't change it
from sqlalchemy.orm import declarative_base

from zvtm.contract.register import register_schema
from zvtm.domain.quotes import StockusKdataCommon

KdataBase = declarative_base()


class Stockus1dKdata(KdataBase, StockusKdataCommon):
    __tablename__ = 'stockus_1d_kdata'


register_schema(providers=['em'], db_name='stockus_1d_kdata', schema_base=KdataBase, entity_type='stockus')

# the __all__ is generated
__all__ = ['Stockus1dKdata']


# -*- coding: utf-8 -*-
# the __all__ is generated
__all__ = []

# __init__.py structure:
# common code of the package
# export interface in __all__ which contains __all__ of its sub modules

# import all from submodule quotes
from .quotes import *
from .quotes import __all__ as _quotes_all
__all__ += _quotes_all

# import all from submodule meta
from .meta import *
from .meta import __all__ as _meta_all
__all__ += _meta_all

# import all from submodule em_api
from .em_api import *
from .em_api import __all__ as _em_api_all
__all__ += _em_api_all

# import all from submodule actor
from .actor import *
from .actor import __all__ as _actor_all
__all__ += _actor_all


# -*- coding: utf-8 -*-

from sqlalchemy.orm import declarative_base

from zvtm.contract import TradableEntity
from zvtm.contract.register import register_schema, register_entity

StockhkMetaBase = declarative_base()


# 港股
@register_entity(entity_type='stockhk')
class Stockhk(StockhkMetaBase, TradableEntity):
    __tablename__ = 'stockhk'


register_schema(providers=['em'], db_name='stockhk_meta',
                schema_base=StockhkMetaBase)
# the __all__ is generated
__all__ = ['Stockhk']


# the __all__ is generated
__all__ = []

# __init__.py structure:
# common code of the package
# export interface in __all__ which contains __all__ of its sub modules

# import all from submodule eastmoney_finance_factor_recorder
from .eastmoney_finance_factor_recorder import *
from .eastmoney_finance_factor_recorder import __all__ as _eastmoney_finance_factor_recorder_all
__all__ += _eastmoney_finance_factor_recorder_all

# import all from submodule eastmoney_income_statement_recorder
from .eastmoney_income_statement_recorder import *
from .eastmoney_income_statement_recorder import __all__ as _eastmoney_income_statement_recorder_all
__all__ += _eastmoney_income_statement_recorder_all

# import all from submodule base_china_stock_finance_recorder
from .base_china_stock_finance_recorder import *
from .base_china_stock_finance_recorder import __all__ as _base_china_stock_finance_recorder_all
__all__ += _base_china_stock_finance_recorder_all

# import all from submodule eastmoney_balance_sheet_recorder
from .eastmoney_balance_sheet_recorder import *
from .eastmoney_balance_sheet_recorder import __all__ as _eastmoney_balance_sheet_recorder_all
__all__ += _eastmoney_balance_sheet_recorder_all

# import all from submodule eastmoney_cash_flow_recorder
from .eastmoney_cash_flow_recorder import *
from .eastmoney_cash_flow_recorder import __all__ as _eastmoney_cash_flow_recorder_all
__all__ += _eastmoney_cash_flow_recorder_all


# the __all__ is generated
__all__ = []

# __init__.py structure:
# common code of the package
# export interface in __all__ which contains __all__ of its sub modules

# import all from submodule zen_factor
from .zen_factor import *
from .zen_factor import __all__ as _zen_factor_all
__all__ += _zen_factor_all

# import all from submodule domain
from .domain import *
from .domain import __all__ as _domain_all
__all__ += _domain_all


# -*- coding: utf-8 -*-
from sqlalchemy import Column, String, DateTime, Float, Integer
from sqlalchemy.orm import declarative_base

from zvtm.contract import Mixin
from zvtm.contract.register import register_schema

FinanceBase = declarative_base()


class BalanceSheet(FinanceBase, Mixin):

    @classmethod
    def important_cols(cls):
        return ['total_assets', 'total_liabilities', 'equity', 'cash_and_cash_equivalents', 'accounts_receivable',
                'inventories', 'goodwill']

    __tablename__ = 'balance_sheet'

    provider = Column(String(length=32))
    code = Column(String(length=32))

    report_period = Column(String(length=32))
    report_date = Column(DateTime)

    # 流动资产
    #
    # 货币资金
    cash_and_cash_equivalents = Column(Float)
    # 应收票据
    note_receivable = Column(Float)
    # 应收账款
    accounts_receivable = Column(Float)
    # 预付款项
    advances_to_suppliers = Column(Float)
    # 其他应收款
    other_receivables = Column(Float)
    # 存货
    inventories = Column(Float)
    # 一年内到期的非流动资产
    current_portion_of_non_current_assets = Column(Float)
    # 其他流动资产
    other_current_assets = Column(Float)
    # 流动资产合计
    total_current_assets = Column(Float)
    # 非流动资产
    #
    # 可供出售金融资产
    fi_assets_saleable = Column(Float)
    # 长期应收款
    long_term_receivables = Column(Float)
    # 长期股权投资
    long_term_equity_investment = Column(Float)
    # 投资性房地产
    real_estate_investment = Column(Float)
    # 固定资产
    fixed_assets = Column(Float)
    # 在建工程
    construction_in_process = Column(Float)
    # 无形资产
    intangible_assets = Column(Float)
    # 商誉
    goodwill = Column(Float)
    # 长期待摊费用
    long_term_prepaid_expenses = Column(Float)
    # 递延所得税资产
    deferred_tax_assets = Column(Float)
    # 其他非流动资产
    other_non_current_assets = Column(Float)
    # 非流动资产合计
    total_non_current_assets = Column(Float)
    # 资产总计
    total_assets = Column(Float)
    # 流动负债
    #
    # 短期借款
    short_term_borrowing = Column(Float)
    # 吸收存款及同业存放
    accept_money_deposits = Column(Float)
    # 应付账款
    accounts_payable = Column(Float)
    # 预收款项
    advances_from_customers = Column(Float)
    # 应付职工薪酬
    employee_benefits_payable = Column(Float)
    # 应交税费
    taxes_payable = Column(Float)
    # 应付利息
    interest_payable = Column(Float)
    # 其他应付款
    other_payable = Column(Float)
    # 一年内到期的非流动负债
    current_portion_of_non_current_liabilities = Column(Float)
    # 其他流动负债
    other_current_liabilities = Column(Float)
    # 流动负债合计
    total_current_liabilities = Column(Float)
    # 非流动负债
    #
    # 长期借款
    long_term_borrowing = Column(Float)
    # 长期应付款
    long_term_payable = Column(Float)
    # 递延收益
    deferred_revenue = Column(Float)
    # 递延所得税负债
    deferred_tax_liabilities = Column(Float)
    # 其他非流动负债
    other_non_current_liabilities = Column(Float)
    # 非流动负债合计
    total_non_current_liabilities = Column(Float)
    # 负债合计
    total_liabilities = Column(Float)
    # 所有者权益(或股东权益)
    #
    # 实收资本（或股本）
    capital = Column(Float)
    # 资本公积
    capital_reserve = Column(Float)
    # 专项储备
    special_reserve = Column(Float)
    # 盈余公积
    surplus_reserve = Column(Float)
    # 未分配利润
    undistributed_profits = Column(Float)
    # 归属于母公司股东权益合计
    equity = Column(Float)
    # 少数股东权益
    equity_as_minority_interest = Column(Float)
    # 股东权益合计
    total_equity = Column(Float)
    # 负债和股东权益合计
    total_liabilities_and_equity = Column(Float)

    # 银行相关
    # 资产
    # 现金及存放中央银行款项
    fi_cash_and_deposit_in_central_bank = Column(Float)
    # 存放同业款项
    fi_deposit_in_other_fi = Column(Float)
    # 贵金属
    fi_expensive_metals = Column(Float)
    # 拆出资金
    fi_lending_to_other_fi = Column(Float)
    # 以公允价值计量且其变动计入当期损益的金融资产
    fi_financial_assets_effect_current_income = Column(Float)
    # 衍生金融资产
    fi_financial_derivative_asset = Column(Float)
    # 买入返售金融资产
    fi_buying_sell_back_fi__asset = Column(Float)
    # 应收账款
    #
    # 应收利息
    fi_interest_receivable = Column(Float)
    # 发放贷款及垫款
    fi_disbursing_loans_and_advances = Column(Float)
    # 可供出售金融资产
    #
    # 持有至到期投资
    fi_held_to_maturity_investment = Column(Float)
    # 应收款项类投资
    fi_account_receivable_investment = Column(Float)
    # 投资性房地产
    #
    # 固定资产
    #
    # 无形资产
    #
    # 商誉
    #
    # 递延所得税资产
    #
    # 其他资产
    fi_other_asset = Column(Float)
    # 资产总计
    #

    # 负债
    #
    # 向中央银行借款
    fi_borrowings_from_central_bank = Column(Float)
    # 同业和其他金融机构存放款项
    fi_deposit_from_other_fi = Column(Float)
    # 拆入资金
    fi_borrowings_from_fi = Column(Float)
    # 以公允价值计量且其变动计入当期损益的金融负债
    fi_financial_liability_effect_current_income = Column(Float)
    # 衍生金融负债
    fi_financial_derivative_liability = Column(Float)
    # 卖出回购金融资产款
    fi_sell_buy_back_fi_asset = Column(Float)
    # 吸收存款
    fi_savings_absorption = Column(Float)
    # 存款证及应付票据
    fi_notes_payable = Column(Float)
    # 应付职工薪酬
    #
    # 应交税费
    #
    # 应付利息
    #
    # 预计负债
    fi_estimated_liabilities = Column(Float)
    # 应付债券
    fi_bond_payable = Column(Float)
    # 其他负债
    fi_other_liability = Column(Float)
    # 负债合计
    #

    # 所有者权益(或股东权益)
    # 股本
    fi_capital = Column(Float)
    # 其他权益工具
    fi_other_equity_instruments = Column(Float)
    # 其中:优先股
    fi_preferred_stock = Column(Float)
    # 资本公积
    #
    # 盈余公积
    #
    # 一般风险准备
    fi_generic_risk_reserve = Column(Float)
    # 未分配利润
    #
    # 归属于母公司股东权益合计
    #
    # 股东权益合计
    #
    # 负债及股东权益总计

    # 券商相关
    # 资产
    #
    # 货币资金
    #
    # 其中: 客户资金存款
    fi_client_fund = Column(Float)
    # 结算备付金
    fi_deposit_reservation_for_balance = Column(Float)
    # 其中: 客户备付金
    fi_client_deposit_reservation_for_balance = Column(Float)
    # 融出资金
    fi_margin_out_fund = Column(Float)
    # 以公允价值计量且其变动计入当期损益的金融资产
    #
    # 衍生金融资产
    #
    # 买入返售金融资产
    #
    # 应收利息
    #
    # 应收款项
    fi_receivables = Column(Float)
    # 存出保证金
    fi_deposit_for_recognizance = Column(Float)
    # 可供出售金融资产
    #
    # 持有至到期投资
    #
    # 长期股权投资
    #
    # 固定资产
    #
    # 在建工程
    #
    # 无形资产
    #
    # 商誉
    #
    # 递延所得税资产
    #
    # 其他资产
    #
    # 资产总计
    #
    # 负债
    #
    # 短期借款
    #
    # 拆入资金
    #
    # 以公允价值计量且其变动计入当期损益的金融负债
    #
    # 衍生金融负债
    #
    # 卖出回购金融资产款
    #
    # 代理买卖证券款
    fi_receiving_as_agent = Column(Float)
    # 应付账款
    #
    # 应付职工薪酬
    #
    # 应交税费
    #
    # 应付利息
    #
    # 应付短期融资款
    fi_short_financing_payable = Column(Float)
    # 预计负债
    #
    # 应付债券
    #
    # 递延所得税负债
    #
    # 其他负债
    #
    # 负债合计
    #
    # 所有者权益(或股东权益)
    #
    # 股本
    #
    # 资本公积
    #
    # 其他权益工具
    #
    # 盈余公积
    #
    # 一般风险准备
    #
    # 交易风险准备
    fi_trade_risk_reserve = Column(Float)
    # 未分配利润
    #
    # 归属于母公司股东权益合计
    #
    # 少数股东权益
    #
    # 股东权益合计
    #
    # 负债和股东权益总计

    # 保险相关

    # 资产
    # 应收保费
    fi_premiums_receivable = Column(Float)
    # 应收分保账款
    fi_reinsurance_premium_receivable = Column(Float)
    # 应收分保合同准备金
    fi_reinsurance_contract_reserve = Column(Float)
    # 保户质押贷款
    fi_policy_pledge_loans = Column(Float)
    # 发放贷款及垫款
    # 定期存款
    fi_time_deposit = Column(Float)
    # 可供出售金融资产
    #
    # 持有至到期投资
    #
    # 应收款项类投资
    #
    # 应收账款
    #
    # 长期股权投资
    #
    # 存出资本保证金
    fi_deposit_for_capital_recognizance = Column(Float)
    # 投资性房地产
    #
    # 固定资产
    #
    # 无形资产
    #
    # 商誉
    #
    # 递延所得税资产
    #
    # 其他资产
    #
    # 独立账户资产
    fi_capital_in_independent_accounts = Column(Float)
    # 资产总计
    #
    # 负债
    #
    # 短期借款
    #
    # 同业及其他金融机构存放款项
    #
    # 拆入资金
    #
    # 以公允价值计量且其变动计入当期损益的金融负债
    #
    # 衍生金融负债
    #
    # 卖出回购金融资产款
    #
    # 吸收存款
    #
    # 代理买卖证券款
    #
    # 应付账款
    #
    # 预收账款
    fi_advance_from_customers = Column(Float)
    # 预收保费
    fi_advance_premium = Column(Float)
    # 应付手续费及佣金
    fi_fees_and_commissions_payable = Column(Float)
    # 应付分保账款
    fi_dividend_payable_for_reinsurance = Column(Float)
    # 应付职工薪酬
    #
    # 应交税费
    #
    # 应付利息
    #
    # 预计负债
    #
    # 应付赔付款
    fi_claims_payable = Column(Float)
    # 应付保单红利
    fi_policy_holder_dividend_payable = Column(Float)
    # 保户储金及投资款
    fi_policy_holder_deposits_and_investment_funds = Column(Float)
    # 保险合同准备金
    fi_contract_reserve = Column(Float)
    # 长期借款
    #
    # 应付债券
    #
    # 递延所得税负债
    #
    # 其他负债
    #
    # 独立账户负债
    fi_independent_liability = Column(Float)
    # 负债合计
    #
    # 所有者权益(或股东权益)
    #
    # 股本
    #
    # 资本公积
    #
    # 盈余公积
    #
    # 一般风险准备
    #
    # 未分配利润
    #
    # 归属于母公司股东权益总计
    #
    # 少数股东权益
    #
    # 股东权益合计
    #
    # 负债和股东权益总计


class IncomeStatement(FinanceBase, Mixin):

    @classmethod
    def important_cols(cls):
        return ['operating_income', 'investment_income', 'total_operating_costs', 'total_profits', 'sales_costs',
                'managing_costs', 'financing_costs']

    __tablename__ = 'income_statement'

    provider = Column(String(length=32))
    code = Column(String(length=32))

    report_period = Column(String(length=32))
    report_date = Column(DateTime)

    # 营业总收入
    #
    # 营业收入
    operating_income = Column(Float)
    # 营业总成本
    total_operating_costs = Column(Float)
    # 营业成本
    operating_costs = Column(Float)
    # 研发费用
    rd_costs = Column(Float)
    # 提取保险合同准备金净额
    net_change_in_insurance_contract_reserves = Column(Float)
    # 营业税金及附加
    business_taxes_and_surcharges = Column(Float)
    # 销售费用
    sales_costs = Column(Float)
    # 管理费用
    managing_costs = Column(Float)
    # 财务费用
    financing_costs = Column(Float)
    # 资产减值损失
    assets_devaluation = Column(Float)
    # 其他经营收益
    #
    # 加: 投资收益
    investment_income = Column(Float)
    # 其中: 对联营企业和合营企业的投资收益
    investment_income_from_related_enterprise = Column(Float)
    # 营业利润
    operating_profit = Column(Float)
    # 加: 营业外收入
    non_operating_income = Column(Float)
    # 减: 营业外支出
    non_operating_costs = Column(Float)
    # 其中: 非流动资产处置净损失
    loss_on_disposal_non_current_asset = Column(Float)

    # 利润总额
    total_profits = Column(Float)
    # 减: 所得税费用
    tax_expense = Column(Float)
    # 净利润
    net_profit = Column(Float)
    # 其中: 归属于母公司股东的净利润
    net_profit_as_parent = Column(Float)
    # 少数股东损益
    net_profit_as_minority_interest = Column(Float)
    # 扣除非经常性损益后的净利润
    deducted_net_profit = Column(Float)
    # 每股收益
    # 基本每股收益
    eps = Column(Float)
    # 稀释每股收益
    diluted_eps = Column(Float)
    # 其他综合收益
    other_comprehensive_income = Column(Float)
    # 归属于母公司股东的其他综合收益
    other_comprehensive_income_as_parent = Column(Float)
    # 归属于少数股东的其他综合收益
    other_comprehensive_income_as_minority_interest = Column(Float)
    # 综合收益总额
    total_comprehensive_income = Column(Float)
    # 归属于母公司所有者的综合收益总额
    total_comprehensive_income_as_parent = Column(Float)
    # 归属于少数股东的综合收益总额
    total_comprehensive_income_as_minority_interest = Column(Float)

    # 银行相关
    # 利息净收入
    fi_net_interest_income = Column(Float)
    # 其中:利息收入
    fi_interest_income = Column(Float)
    # 利息支出
    fi_interest_expenses = Column(Float)
    # 手续费及佣金净收入
    fi_net_incomes_from_fees_and_commissions = Column(Float)
    # 其中:手续费及佣金收入
    fi_incomes_from_fees_and_commissions = Column(Float)
    # 手续费及佣金支出
    fi_expenses_for_fees_and_commissions = Column(Float)
    # 公允价值变动收益
    fi_income_from_fair_value_change = Column(Float)
    # 汇兑收益
    fi_income_from_exchange = Column(Float)
    # 其他业务收入
    fi_other_income = Column(Float)
    # 业务及管理费
    fi_operate_and_manage_expenses = Column(Float)

    # 保险相关
    # 已赚保费
    fi_net_income_from_premium = Column(Float)
    # 其中:保险业务收入
    fi_income_from_premium = Column(Float)
    # 分保费收入
    fi_income_from_reinsurance_premium = Column(Float)
    # 减:分出保费
    fi_reinsurance_premium = Column(Float)
    # 提取未到期责任准备金
    fi_undue_duty_reserve = Column(Float)
    # 银行业务利息净收入
    fi_net_income_from_bank_interest = Column(Float)
    # 其中:银行业务利息收入
    fi_income_from_bank_interest = Column(Float)
    # 银行业务利息支出
    fi_expenses_for_bank_interest = Column(Float)
    # 非保险业务手续费及佣金净收入
    fi_net_incomes_from_fees_and_commissions_of_non_insurance = Column(Float)
    # 非保险业务手续费及佣金收入
    fi_incomes_from_fees_and_commissions_of_non_insurance = Column(Float)
    # 非保险业务手续费及佣金支出
    fi_expenses_for_fees_and_commissions_of_non_insurance = Column(Float)
    # 退保金
    fi_insurance_surrender_costs = Column(Float)
    # 赔付支出
    fi_insurance_claims_expenses = Column(Float)
    # 减:摊回赔付支出
    fi_amortized_insurance_claims_expenses = Column(Float)
    # 提取保险责任准备金
    fi_insurance_duty_reserve = Column(Float)
    # 减:摊回保险责任准备金
    fi_amortized_insurance_duty_reserve = Column(Float)
    # 保单红利支出
    fi_dividend_expenses_to_insured = Column(Float)
    # 分保费用
    fi_reinsurance_expenses = Column(Float)
    # 减:摊回分保费用
    fi_amortized_reinsurance_expenses = Column(Float)
    # 其他业务成本
    fi_other_op_expenses = Column(Float)

    # 券商相关
    # 手续费及佣金净收入
    #
    # 其中:代理买卖证券业务净收入
    fi_net_incomes_from_trading_agent = Column(Float)
    # 证券承销业务净收入
    fi_net_incomes_from_underwriting = Column(Float)
    # 受托客户资产管理业务净收入
    fi_net_incomes_from_customer_asset_management = Column(Float)
    # 手续费及佣金净收入其他项目
    fi_fees_from_other = Column(Float)
    # 公允价值变动收益
    #
    # 其中:可供出售金融资产公允价值变动损益
    fi_income_from_fair_value_change_of_fi_salable = Column(Float)


class CashFlowStatement(FinanceBase, Mixin):
    @classmethod
    def important_cols(cls):
        return ['net_op_cash_flows', 'net_investing_cash_flows', 'net_financing_cash_flows', 'cash']

    __tablename__ = 'cash_flow_statement'

    provider = Column(String(length=32))
    code = Column(String(length=32))

    report_period = Column(String(length=32))
    report_date = Column(DateTime)
    # 经营活动产生的现金流量
    #
    # 销售商品、提供劳务收到的现金
    cash_from_selling = Column(Float)

    # 收到的税费返还
    tax_refund = Column(Float)

    # 收到其他与经营活动有关的现金
    cash_from_other_op = Column(Float)

    # 经营活动现金流入小计
    total_op_cash_inflows = Column(Float)

    # 购买商品、接受劳务支付的现金
    cash_to_goods_services = Column(Float)
    # 支付给职工以及为职工支付的现金
    cash_to_employees = Column(Float)
    # 支付的各项税费
    taxes_and_surcharges = Column(Float)
    # 支付其他与经营活动有关的现金
    cash_to_other_related_op = Column(Float)
    # 经营活动现金流出小计
    total_op_cash_outflows = Column(Float)

    # 经营活动产生的现金流量净额
    net_op_cash_flows = Column(Float)

    # 投资活动产生的现金流量

    # 收回投资收到的现金
    cash_from_disposal_of_investments = Column(Float)
    # 取得投资收益收到的现金
    cash_from_returns_on_investments = Column(Float)
    # 处置固定资产、无形资产和其他长期资产收回的现金净额
    cash_from_disposal_fixed_intangible_assets = Column(Float)
    # 处置子公司及其他营业单位收到的现金净额
    cash_from_disposal_subsidiaries = Column(Float)

    # 收到其他与投资活动有关的现金
    cash_from_other_investing = Column(Float)

    # 投资活动现金流入小计
    total_investing_cash_inflows = Column(Float)

    # 购建固定资产、无形资产和其他长期资产支付的现金
    cash_to_acquire_fixed_intangible_assets = Column(Float)
    # 投资支付的现金
    cash_to_investments = Column(Float)

    # 取得子公司及其他营业单位支付的现金净额
    cash_to_acquire_subsidiaries = Column(Float)

    # 支付其他与投资活动有关的现金
    cash_to_other_investing = Column(Float)

    # 投资活动现金流出小计
    total_investing_cash_outflows = Column(Float)

    # 投资活动产生的现金流量净额
    net_investing_cash_flows = Column(Float)

    # 筹资活动产生的现金流量
    #
    # 吸收投资收到的现金
    cash_from_accepting_investment = Column(Float)
    # 子公司吸收少数股东投资收到的现金
    cash_from_subsidiaries_accepting_minority_interest = Column(Float)

    # 取得借款收到的现金
    cash_from_borrowings = Column(Float)
    # 发行债券收到的现金
    cash_from_issuing_bonds = Column(Float)
    # 收到其他与筹资活动有关的现金
    cash_from_other_financing = Column(Float)

    # 筹资活动现金流入小计
    total_financing_cash_inflows = Column(Float)

    # 偿还债务支付的现金
    cash_to_repay_borrowings = Column(Float)

    # 分配股利、利润或偿付利息支付的现金
    cash_to_pay_interest_dividend = Column(Float)

    # 子公司支付给少数股东的股利、利润
    cash_to_pay_subsidiaries_minority_interest = Column(Float)

    # 支付其他与筹资活动有关的现金
    cash_to_other_financing = Column(Float)
    # 筹资活动现金流出小计
    total_financing_cash_outflows = Column(Float)

    # 筹资活动产生的现金流量净额
    net_financing_cash_flows = Column(Float)
    # 汇率变动对现金及现金等价物的影响
    foreign_exchange_rate_effect = Column(Float)
    # 现金及现金等价物净增加额
    net_cash_increase = Column(Float)
    # 加: 期初现金及现金等价物余额
    cash_at_beginning = Column(Float)
    # 期末现金及现金等价物余额
    cash = Column(Float)

    # 银行相关
    # 客户存款和同业及其他金融机构存放款项净增加额
    fi_deposit_increase = Column(Float)
    # 向中央银行借款净增加额
    fi_borrow_from_central_bank_increase = Column(Float)
    # 存放中央银行和同业款项及其他金融机构净减少额
    fi_deposit_in_others_decrease = Column(Float)
    # 拆入资金及卖出回购金融资产款净增加额
    fi_borrowing_and_sell_repurchase_increase = Column(Float)
    # 其中:卖出回购金融资产款净增加额
    fi_sell_repurchase_increase = Column(Float)
    # 拆出资金及买入返售金融资产净减少额
    fi_lending_and_buy_repurchase_decrease = Column(Float)
    # 其中:拆出资金净减少额
    fi_lending_decrease = Column(Float)
    # 买入返售金融资产净减少额
    fi_buy_repurchase_decrease = Column(Float)
    # 收取的利息、手续费及佣金的现金
    fi_cash_from_interest_commission = Column(Float)
    # 客户贷款及垫款净增加额
    fi_loan_advance_increase = Column(Float)
    # 存放中央银行和同业及其他金融机构款项净增加额
    fi_deposit_in_others_increase = Column(Float)
    # 拆出资金及买入返售金融资产净增加额
    fi_lending_and_buy_repurchase_increase = Column(Float)
    # 其中:拆出资金净增加额
    fi_lending_increase = Column(Float)
    # 拆入资金及卖出回购金融资产款净减少额
    fi_borrowing_and_sell_repurchase_decrease = Column(Float)
    # 其中:拆入资金净减少额
    fi_borrowing_decrease = Column(Float)
    # 卖出回购金融资产净减少额
    fi_sell_repurchase_decrease = Column(Float)
    # 支付利息、手续费及佣金的现金
    fi_cash_to_interest_commission = Column(Float)
    # 应收账款净增加额
    fi_account_receivable_increase = Column(Float)
    # 偿付债券利息支付的现金
    fi_cash_to_pay_interest = Column(Float)

    # 保险相关
    # 收到原保险合同保费取得的现金
    fi_cash_from_premium_of_original = Column(Float)
    # 保户储金及投资款净增加额
    fi_insured_deposit_increase = Column(Float)
    # 银行及证券业务卖出回购资金净增加额
    fi_bank_broker_sell_repurchase_increase = Column(Float)
    # 银行及证券业务买入返售资金净减少额
    fi_bank_broker_buy_repurchase_decrease = Column(Float)
    # 支付原保险合同赔付等款项的现金
    fi_cash_to_insurance_claim = Column(Float)
    # 支付再保险业务现金净额
    fi_cash_to_reinsurance = Column(Float)
    # 银行业务及证券业务拆借资金净减少额
    fi_lending_decrease = Column(Float)
    # 银行业务及证券业务卖出回购资金净减少额
    fi_bank_broker_sell_repurchase_decrease = Column(Float)
    # 支付保单红利的现金
    fi_cash_to_dividends = Column(Float)
    # 保户质押贷款净增加额
    fi_insured_pledge_loans_increase = Column(Float)
    # 收购子公司及其他营业单位支付的现金净额
    fi_cash_to_acquire_subsidiaries = Column(Float)
    # 处置子公司及其他营业单位流出的现金净额
    fi_cash_to_disposal_subsidiaries = Column(Float)
    # 支付卖出回购金融资产款现金净额
    fi_cash_to_sell_repurchase = Column(Float)

    # 券商相关
    # 拆入资金净增加额
    fi_borrowing_increase = Column(Float)
    # 代理买卖证券收到的现金净额
    fi_cash_from_trading_agent = Column(Float)
    # 回购业务资金净增加额
    fi_cash_from_repurchase_increase = Column(Float)
    # 处置交易性金融资产的净减少额
    fi_disposal_trade_asset_decrease = Column(Float)
    # 回购业务资金净减少额
    fi_repurchase_decrease = Column(Float)
    # 代理买卖证券支付的现金净额（净减少额）
    fi_cash_to_agent_trade = Column(Float)


# 主要财务指标

class FinanceFactor(FinanceBase, Mixin):
    @classmethod
    def important_cols(cls):
        return ['basic_eps', 'total_op_income', 'net_profit', 'op_income_growth_yoy', 'net_profit_growth_yoy', 'roe',
                'rota', 'gross_profit_margin', 'net_margin']

    __tablename__ = 'finance_factor'

    provider = Column(String(length=32))
    code = Column(String(length=32))

    report_period = Column(String(length=32))
    report_date = Column(DateTime)
    # 每股指标
    #
    # 基本每股收益(元)
    basic_eps = Column(Float)
    # 扣非每股收益(元)
    deducted_eps = Column(Float)
    # 稀释每股收益(元)
    diluted_eps = Column(Float)
    # 每股净资产(元)
    bps = Column(Float)
    # 每股资本公积(元)
    capital_reserve_ps = Column(Float)
    # 每股未分配利润(元)
    undistributed_profit_ps = Column(Float)
    # 每股经营现金流(元)
    op_cash_flow_ps = Column(Float)
    # 成长能力指标
    #
    # 营业总收入(元)
    total_op_income = Column(Float)
    # 毛利润(元)
    gross_profit = Column(Float)
    # 归属净利润(元)
    net_profit = Column(Float)
    # 扣非净利润(元)
    deducted_net_profit = Column(Float)
    # 营业总收入同比增长
    op_income_growth_yoy = Column(Float)
    # 归属净利润同比增长
    net_profit_growth_yoy = Column(Float)
    # 扣非净利润同比增长
    deducted_net_profit_growth_yoy = Column(Float)
    # 营业总收入滚动环比增长
    op_income_growth_qoq = Column(Float)
    # 归属净利润滚动环比增长
    net_profit_growth_qoq = Column(Float)
    # 扣非净利润滚动环比增长
    deducted_net_profit_growth_qoq = Column(Float)
    # 盈利能力指标
    #
    # 净资产收益率(加权)
    roe = Column(Float)
    # 净资产收益率(扣非/加权)
    deducted_roe = Column(Float)
    # 总资产收益率(加权)
    rota = Column(Float)
    # 毛利率
    gross_profit_margin = Column(Float)
    # 净利率
    net_margin = Column(Float)
    # 收益质量指标
    #
    # 预收账款/营业收入
    advance_receipts_per_op_income = Column(Float)
    # 销售净现金流/营业收入
    sales_net_cash_flow_per_op_income = Column(Float)
    # 经营净现金流/营业收入
    op_net_cash_flow_per_op_income = Column(Float)
    # 实际税率
    actual_tax_rate = Column(Float)
    # 财务风险指标
    #
    # 流动比率
    current_ratio = Column(Float)
    # 速动比率
    quick_ratio = Column(Float)
    # 现金流量比率
    cash_flow_ratio = Column(Float)
    # 资产负债率
    debt_asset_ratio = Column(Float)
    # 权益乘数
    em = Column(Float)
    # 产权比率
    equity_ratio = Column(Float)
    # 营运能力指标(一般企业)
    #
    # 总资产周转天数(天)
    total_assets_turnover_days = Column(Integer)
    # 存货周转天数(天)
    inventory_turnover_days = Column(Integer)
    # 应收账款周转天数(天)
    receivables_turnover_days = Column(Integer)
    # 总资产周转率(次)
    total_assets_turnover = Column(Float)
    # 存货周转率(次)
    inventory_turnover = Column(Float)
    # 应收账款周转率(次)
    receivables_turnover = Column(Float)

    # 专项指标(银行)
    #
    # 存款总额
    fi_total_deposit = Column(Float)
    # 贷款总额
    fi_total_loan = Column(Float)
    # 存贷款比例
    fi_loan_deposit_ratio = Column(Float)
    # 资本充足率
    fi_capital_adequacy_ratio = Column(Float)
    # 核心资本充足率
    fi_core_capital_adequacy_ratio = Column(Float)
    # 不良贷款率
    fi_npl_ratio = Column(Float)
    # 不良贷款拨备覆盖率
    fi_npl_provision_coverage = Column(Float)
    # 资本净额
    fi_net_capital = Column(Float)
    # 专项指标(保险)
    #
    # 总投资收益率
    insurance_roi = Column(Float)
    # 净投资收益率
    insurance_net_investment_yield = Column(Float)
    # 已赚保费
    insurance_earned_premium = Column(Float)
    # 赔付支出
    insurance_payout = Column(Float)
    # 退保率
    insurance_surrender_rate = Column(Float)
    # 偿付能力充足率
    insurance_solvency_adequacy_ratio = Column(Float)
    # 专项指标(券商)
    #
    # 净资本
    broker_net_capital = Column(Float)
    # 净资产
    broker_net_assets = Column(Float)
    # 净资本/净资产
    broker_net_capital_assets_ratio = Column(Float)
    # 自营固定收益类证券规模/净资本
    broker_self_operated_fixed_income_securities_net_capital_ratio = Column(Float)


register_schema(providers=['eastmoney'], db_name='finance', schema_base=FinanceBase, entity_type='stock')

# the __all__ is generated
__all__ = ['BalanceSheet', 'IncomeStatement', 'CashFlowStatement', 'FinanceFactor']


# -*- coding: utf-8 -*-
import logging

import requests

from zvtm.contract.api import get_data_count, get_data
from zvtm.contract.recorder import TimestampsDataRecorder, TimeSeriesDataRecorder
from zvtm.domain import CompanyType
from zvtm.domain.meta.stock_meta import Stock, StockDetail
from zvtm.utils.time_utils import to_pd_timestamp

logger = logging.getLogger(__name__)


class ApiWrapper(object):
    def request(self, url=None, method='post', param=None, path_fields=None):
        raise NotImplementedError


def get_fc(security_item):
    if security_item.exchange == 'sh':
        fc = "{}01".format(security_item.code)
    if security_item.exchange == 'sz':
        fc = "{}02".format(security_item.code)

    return fc


def get_company_type(stock_domain: StockDetail):
    industries = stock_domain.industries.split(',')
    if ('银行' in industries) or ('信托' in industries):
        return CompanyType.yinhang
    if '保险' in industries:
        return CompanyType.baoxian
    if '证券' in industries:
        return CompanyType.quanshang
    return CompanyType.qiye


def company_type_flag(security_item):
    try:
        company_type = get_company_type(security_item)

        if company_type == CompanyType.qiye:
            return "4"
        if company_type == CompanyType.quanshang:
            return "1"
        if company_type == CompanyType.baoxian:
            return "2"
        if company_type == CompanyType.yinhang:
            return "3"
    except Exception as e:
        logger.warning(e)

    param = {
        "color": "w",
        "fc": get_fc(security_item)
    }

    resp = requests.post('https://emh5.eastmoney.com/api/CaiWuFenXi/GetCompanyType', json=param)

    ct = resp.json().get('Result').get('CompanyType')

    logger.warning("{} not catching company type:{}".format(security_item, ct))

    return ct


def call_eastmoney_api(url=None, method='post', param=None, path_fields=None):
    if method == 'post':
        resp = requests.post(url, json=param)

    resp.encoding = 'utf8'

    try:
        origin_result = resp.json().get('Result')
    except Exception as e:
        logger.exception('code:{},content:{}'.format(resp.status_code, resp.text))
        raise e

    if path_fields:
        the_data = get_from_path_fields(origin_result, path_fields)
        if not the_data:
            logger.warning(
                "url:{},param:{},origin_result:{},could not get data for nested_fields:{}".format(url, param,
                                                                                                  origin_result,
                                                                                                  path_fields))
        return the_data

    return origin_result


def get_from_path_fields(the_json, path_fields):
    the_data = the_json.get(path_fields[0])
    if the_data:
        for field in path_fields[1:]:
            the_data = the_data.get(field)
            if not the_data:
                return None
    return the_data


class EastmoneyApiWrapper(ApiWrapper):
    def request(self, url=None, method='post', param=None, path_fields=None):
        return call_eastmoney_api(url=url, method=method, param=param, path_fields=path_fields)


class BaseEastmoneyRecorder(object):
    request_method = 'post'
    path_fields = None
    api_wrapper = EastmoneyApiWrapper()

    def generate_request_param(self, security_item, start, end, size, timestamp):
        raise NotImplementedError

    def record(self, entity_item, start, end, size, timestamps):
        if timestamps:
            original_list = []
            for the_timestamp in timestamps:
                param = self.generate_request_param(entity_item, start, end, size, the_timestamp)
                tmp_list = self.api_wrapper.request(url=self.url, param=param, method=self.request_method,
                                                    path_fields=self.path_fields)
                self.logger.info(
                    "record {} for entity_id:{},timestamp:{}".format(
                        self.data_schema, entity_item.id, the_timestamp))
                # fill timestamp field
                for tmp in tmp_list:
                    tmp[self.get_evaluated_time_field()] = the_timestamp
                original_list += tmp_list
                if len(original_list) == 50:
                    break
            return original_list

        else:
            param = self.generate_request_param(entity_item, start, end, size, None)
            return self.api_wrapper.request(url=self.url, param=param, method=self.request_method,
                                            path_fields=self.path_fields)


class EastmoneyTimestampsDataRecorder(BaseEastmoneyRecorder, TimestampsDataRecorder):
    entity_provider = 'eastmoney'
    entity_schema = StockDetail

    provider = 'eastmoney'

    timestamps_fetching_url = None
    timestamp_list_path_fields = None
    timestamp_path_fields = None

    def init_timestamps(self, entity):
        param = {
            "color": "w",
            "fc": get_fc(entity)
        }

        timestamp_json_list = call_eastmoney_api(url=self.timestamps_fetching_url,
                                                 path_fields=self.timestamp_list_path_fields,
                                                 param=param)

        if self.timestamp_path_fields and timestamp_json_list:
            timestamps = [get_from_path_fields(data, self.timestamp_path_fields) for data in timestamp_json_list]
            return [to_pd_timestamp(t) for t in timestamps]
        return []


class EastmoneyPageabeDataRecorder(BaseEastmoneyRecorder, TimeSeriesDataRecorder):
    entity_provider = 'eastmoney'
    entity_schema = StockDetail

    provider = 'eastmoney'

    page_url = None

    def get_remote_count(self, security_item):
        param = {
            "color": "w",
            "fc": get_fc(security_item),
            "pageNum": 1,
            "pageSize": 1
        }
        return call_eastmoney_api(self.page_url, param=param, path_fields=['TotalCount'])

    def evaluate_start_end_size_timestamps(self, entity):
        remote_count = self.get_remote_count(entity)

        if remote_count == 0:
            return None, None, 0, None

        # get local count
        local_count = get_data_count(data_schema=self.data_schema, session=self.session,
                                     filters=[self.data_schema.entity_id == entity.id])
        # FIXME:the > case
        if local_count >= remote_count:
            return None, None, 0, None

        return None, None, remote_count - local_count, None

    def generate_request_param(self, security_item, start, end, size, timestamp):
        return {
            "color": "w",
            "fc": get_fc(security_item),
            'pageNum': 1,
            # just get more for some fixed data
            'pageSize': size + 10
        }


class EastmoneyMoreDataRecorder(BaseEastmoneyRecorder, TimeSeriesDataRecorder):
    entity_provider = 'eastmoney'
    entity_schema = StockDetail

    provider = 'eastmoney'

    def get_remote_latest_record(self, security_item):
        param = {
            "color": "w",
            "fc": get_fc(security_item),
            "pageNum": 1,
            "pageSize": 1
        }
        results = call_eastmoney_api(self.url, param=param, path_fields=self.path_fields)
        _, result = self.generate_domain(security_item, results[0])
        return result

    def evaluate_start_end_size_timestamps(self, entity):
        # get latest record
        latest_record = get_data(entity_id=entity.id,
                                 provider=self.provider,
                                 data_schema=self.data_schema,
                                 order=self.data_schema.timestamp.desc(), limit=1,
                                 return_type='domain',
                                 session=self.session)
        if latest_record:
            remote_record = self.get_remote_latest_record(entity)
            if not remote_record or (
                    latest_record[0].id == remote_record.id):
                return None, None, 0, None
            else:
                return None, None, 10, None

        return None, None, 1000, None

    def generate_request_param(self, security_item, start, end, size, timestamp):
        return {
            "color": "w",
            "fc": get_fc(security_item),
            'pageNum': 1,
            'pageSize': size
        }
# the __all__ is generated
__all__ = ['ApiWrapper', 'get_fc', 'get_company_type', 'company_type_flag', 'call_eastmoney_api', 'get_from_path_fields', 'EastmoneyApiWrapper', 'BaseEastmoneyRecorder', 'EastmoneyTimestampsDataRecorder', 'EastmoneyPageabeDataRecorder', 'EastmoneyMoreDataRecorder']


# -*- coding: utf-8 -*-
# the __all__ is generated
__all__ = []

# __init__.py structure:
# common code of the package
# export interface in __all__ which contains __all__ of its sub modules

# import all from submodule macd_factor
from .macd_factor import *
from .macd_factor import __all__ as _macd_factor_all
__all__ += _macd_factor_all


# -*- coding: utf-8 -*-

import pandas as pd
from jqdatapy.api import get_mtss

from zvtm.contract.api import df_to_db
from zvtm.contract.recorder import TimeSeriesDataRecorder
from zvtm.domain import Stock, MarginTrading
from zvtm.recorders.joinquant.common import to_jq_entity_id
from zvtm.utils.pd_utils import pd_is_not_null
from zvtm.utils.time_utils import to_time_str, TIME_FORMAT_DAY


class MarginTradingRecorder(TimeSeriesDataRecorder):
    entity_provider = 'joinquant'
    entity_schema = Stock

    # 数据来自jq
    provider = 'joinquant'

    data_schema = MarginTrading

    def record(self, entity, start, end, size, timestamps):
        df = get_mtss(code=to_jq_entity_id(entity), date=to_time_str(start))

        if pd_is_not_null(df):
            df['entity_id'] = entity.id
            df['code'] = entity.code
            df.rename(columns={'date': 'timestamp'}, inplace=True)
            df['timestamp'] = pd.to_datetime(df['timestamp'])
            df['id'] = df[['entity_id', 'timestamp']].apply(
                lambda se: "{}_{}".format(se['entity_id'], to_time_str(se['timestamp'], fmt=TIME_FORMAT_DAY)), axis=1)

            print(df)
            df_to_db(df=df, data_schema=self.data_schema, provider=self.provider, force_update=self.force_update)

        return None


if __name__ == '__main__':
    MarginTradingRecorder(codes=['000004']).run()
# the __all__ is generated
__all__ = ['MarginTradingRecorder']


# -*- coding: utf-8 -*-
import enum
import json
import logging
import time
from typing import List, Union, Optional, Type

import pandas as pd
from sqlalchemy import Column, String, Text
from sqlalchemy.orm import declarative_base

from zvtm.contract import IntervalLevel, TradableEntity
from zvtm.contract import Mixin
from zvtm.contract.api import get_data, df_to_db, get_db_session, del_data
from zvtm.contract.reader import DataReader, DataListener
from zvtm.contract.register import register_schema
from zvtm.contract.zvt_context import factor_cls_registry
from zvtm.utils.pd_utils import pd_is_not_null


class Indicator(object):
    def __init__(self) -> None:
        self.logger = logging.getLogger(self.__class__.__name__)
        self.indicators = []


class Transformer(Indicator):

    def __init__(self) -> None:
        super().__init__()

    def transform(self, input_df: pd.DataFrame) -> pd.DataFrame:
        """
        input_df format:

                                  col1    col2    col3    ...
        entity_id    timestamp
                                  1.2     0.5     0.3     ...
                                  1.0     0.7     0.2     ...

        the return result would change the columns and  keep the format

        :param input_df:
        :return:
        """
        g = input_df.groupby(level=0)
        if len(g.groups) == 1:
            entity_id = input_df.index[0][0]

            df = input_df.reset_index(level=0, drop=True)
            ret_df = self.transform_one(entity_id=entity_id, df=df)
            ret_df['entity_id'] = entity_id

            return ret_df.set_index('entity_id', append=True).swaplevel(0, 1)
        else:
            return g.apply(lambda x: self.transform_one(x.index[0][0], x.reset_index(level=0, drop=True)))

    def transform_one(self, entity_id, df: pd.DataFrame) -> pd.DataFrame:
        """
        df format:

                     col1    col2    col3    ...
        timestamp
                     1.2     0.5     0.3     ...
                     1.0     0.7     0.2     ...

        the return result would change the columns and  keep the format

        :param df:
        :return:
        """
        return df


class Accumulator(Indicator):

    def __init__(self, acc_window: int = 1) -> None:
        """

        :param acc_window: the window size of acc for computing,default is 1
        """
        super().__init__()
        self.acc_window = acc_window

    def acc(self, input_df: pd.DataFrame, acc_df: pd.DataFrame, states: dict) -> (pd.DataFrame, dict):
        """

        :param input_df: new input
        :param acc_df: previous result
        :param states: current states of the entity
        :return: new result and states
        """
        g = input_df.groupby(level=0)
        if len(g.groups) == 1:
            entity_id = input_df.index[0][0]

            df = input_df.reset_index(level=0, drop=True)
            if pd_is_not_null(acc_df) and (entity_id == acc_df.index[0][0]):
                acc_one_df = acc_df.reset_index(level=0, drop=True)
            else:
                acc_one_df = None
            ret_df, state = self.acc_one(entity_id=entity_id, df=df, acc_df=acc_one_df, state=states.get(entity_id))
            if pd_is_not_null(ret_df):
                ret_df['entity_id'] = entity_id
                ret_df = ret_df.set_index('entity_id', append=True).swaplevel(0, 1)
                ret_df['entity_id'] = entity_id
                return ret_df, {entity_id: state}
            return None, {entity_id: state}
        else:
            new_states = {}

            def cal_acc(x):
                entity_id = x.index[0][0]
                if pd_is_not_null(acc_df):
                    acc_g = acc_df.groupby(level=0)
                    acc_one_df = None
                    if entity_id in acc_g.groups:
                        acc_one_df = acc_g.get_group(entity_id)
                        if pd_is_not_null(acc_one_df):
                            acc_one_df = acc_one_df.reset_index(level=0, drop=True)
                else:
                    acc_one_df = None

                one_result, state = self.acc_one(entity_id=entity_id,
                                                 df=x.reset_index(level=0, drop=True),
                                                 acc_df=acc_one_df,
                                                 state=states.get(x.index[0][0]))

                new_states[entity_id] = state
                return one_result

            ret_df = g.apply(lambda x: cal_acc(x))
            return ret_df, new_states

    def acc_one(self, entity_id, df: pd.DataFrame, acc_df: pd.DataFrame, state: dict) -> (pd.DataFrame, dict):
        """
        df format:

                     col1    col2    col3    ...
        timestamp
                     1.2     0.5     0.3     ...
                     1.0     0.7     0.2     ...

        the new result and state

        :param df: current input df
        :param entity_id: current computing entity_id
        :param acc_df: current result of the entity_id
        :param state: current state of the entity_id
        :return: new result and state of the entity_id
        """
        return acc_df, state


class Scorer(object):
    def __init__(self) -> None:
        self.logger = logging.getLogger(self.__class__.__name__)

    def score(self, input_df: pd.DataFrame) -> pd.DataFrame:
        """

        :param input_df: current input df
        :return: df with normal score
        """
        return input_df


class FactorType(enum.Enum):
    filter = 'filter'
    score = 'score'


def register_class(target_class):
    if target_class.__name__ not in ('Factor', 'FilterFactor', 'ScoreFactor', 'StateFactor'):
        factor_cls_registry[target_class.__name__] = target_class


class FactorMeta(type):
    def __new__(meta, name, bases, class_dict):
        cls = type.__new__(meta, name, bases, class_dict)
        register_class(cls)
        return cls


FactorBase = declarative_base()


# 用于保存factor的状态
class FactorState(FactorBase, Mixin):
    __tablename__ = 'factor_state'
    # 因子名字
    factor_name = Column(String(length=128))

    # json string
    state = Column(Text())


register_schema(providers=['zvt'], db_name='factor_info', schema_base=FactorBase)


class Factor(DataReader, DataListener):
    factor_type: FactorType = None

    # define the schema for persist,its columns should be same as indicators in transformer or accumulator
    factor_schema: Type[Mixin] = None

    transformer: Transformer = None
    accumulator: Accumulator = None

    def __init__(self,
                 data_schema: Type[Mixin],
                 entity_schema: Type[TradableEntity] = None,
                 provider: str = None,
                 entity_provider: str = None,
                 entity_ids: List[str] = None,
                 exchanges: List[str] = None,
                 codes: List[str] = None,
                 start_timestamp: Union[str, pd.Timestamp] = None,
                 end_timestamp: Union[str, pd.Timestamp] = None,
                 columns: List = None,
                 filters: List = None,
                 order: object = None,
                 limit: int = None,
                 level: Union[str, IntervalLevel] = None,
                 category_field: str = 'entity_id',
                 time_field: str = 'timestamp',
                 computing_window: int = None,
                 keep_all_timestamp: bool = False,
                 fill_method: str = 'ffill',
                 effective_number: int = None,
                 transformer: Transformer = None,
                 accumulator: Accumulator = None,
                 need_persist: bool = False,
                 only_compute_factor: bool = False,
                 factor_name: str = None,
                 clear_state: bool = False,
                 only_load_factor: bool = False) -> None:
        """
        :param keep_all_timestamp:
        :param fill_method:
        :param effective_number:
        :param transformer:
        :param accumulator:
        :param need_persist: whether persist factor
        :param only_compute_factor: only compute factor nor result
        :param factor_name:
        :param clear_state:
        :param only_load_factor: only load factor and compute result
        """
        self.only_load_factor = only_load_factor

        super().__init__(data_schema, entity_schema, provider, entity_provider, entity_ids, exchanges, codes,
                         start_timestamp, end_timestamp, columns, filters, order, limit, level,
                         category_field, time_field, computing_window)

        # define unique name of your factor if you want to keep factor state
        # the factor state is defined by factor_name and entity_id
        if not factor_name:
            self.factor_name = type(self).__name__.lower()
        else:
            self.factor_name = factor_name

        self.clear_state = clear_state

        self.keep_all_timestamp = keep_all_timestamp
        self.fill_method = fill_method
        self.effective_number = effective_number

        if transformer:
            self.transformer = transformer
        else:
            self.transformer = self.__class__.transformer

        if accumulator:
            self.accumulator = accumulator
        else:
            self.accumulator = self.__class__.accumulator

        self.need_persist = need_persist
        self.dry_run = only_compute_factor

        # 中间结果，不持久化
        # data_df->pipe_df
        self.pipe_df: pd.DataFrame = None

        # 计算因子的结果，可持久化,通过对pipe_df的计算得到
        # pipe_df->factor_df
        self.factor_df: pd.DataFrame = None

        # result_df是用于选股的标准df,通过对factor_df的计算得到
        # factor_df->result_df
        self.result_df: pd.DataFrame = None

        # entity_id:state
        self.states: dict = {}

        if self.clear_state:
            self.clear_state_data()
        elif self.need_persist or self.only_load_factor:
            self.load_factor()

            # 根据已经计算的factor_df和computing_window来保留data_df
            # 因为读取data_df的目的是为了计算factor_df,选股和回测只依赖factor_df
            # 所以如果有持久化的factor_df,只需保留需要用于计算的data_df即可
            if pd_is_not_null(self.data_df) and self.computing_window:
                dfs = []
                for entity_id, df in self.data_df.groupby(level=0):
                    latest_laved = get_data(provider='zvt',
                                            data_schema=self.factor_schema,
                                            entity_id=entity_id,
                                            order=self.factor_schema.timestamp.desc(),
                                            limit=1,
                                            index=[self.category_field, self.time_field], return_type='domain')
                    if latest_laved:
                        df1 = df[df.timestamp < latest_laved[0].timestamp].iloc[-self.computing_window:]
                        if pd_is_not_null(df1):
                            df = df[df.timestamp >= df1.iloc[0].timestamp]
                    dfs.append(df)

                self.data_df = pd.concat(dfs)

        self.register_data_listener(self)

        # the compute logic is not triggered from load data
        # for the case:1)load factor from db 2)compute the result
        if self.only_load_factor:
            self.compute()

    def load_data(self):
        if self.only_load_factor:
            return
        super().load_data()

    def load_factor(self):
        # read state
        states: List[FactorState] = FactorState.query_data(filters=[FactorState.factor_name == self.factor_name],
                                                           entity_ids=self.entity_ids,
                                                           return_type='domain')
        if states:
            for state in states:
                self.states[state.entity_id] = self.decode_state(state.state)

        if self.dry_run:
            # 如果只是为了计算因子，只需要读取acc_window的factor_df
            if self.accumulator is not None:
                self.factor_df = self.load_window_df(provider='zvt', data_schema=self.factor_schema,
                                                     window=self.accumulator.acc_window)
        else:
            self.factor_df = get_data(provider='zvt',
                                      data_schema=self.factor_schema,
                                      start_timestamp=self.start_timestamp,
                                      entity_ids=self.entity_ids,
                                      end_timestamp=self.end_timestamp,
                                      index=[self.category_field, self.time_field])

        col_map_object_hook = self.factor_col_map_object_hook()
        if pd_is_not_null(self.factor_df) and col_map_object_hook:
            for col in col_map_object_hook:
                if col in self.factor_df.columns:
                    self.factor_df[col] = self.factor_df[col].apply(
                        lambda x: json.loads(x, object_hook=col_map_object_hook.get(col)) if x else None)

    def factor_col_map_object_hook(self) -> dict:
        """

        :return:{col:object_hook}
        """
        return {}

    def factor_state_object_hook(self):
        return None

    def clear_state_data(self, entity_id=None):
        if entity_id:
            del_data(FactorState,
                     filters=[FactorState.factor_name == self.factor_name, FactorState.entity_id == entity_id],
                     provider='zvt')
            del_data(self.factor_schema, filters=[self.factor_schema.entity_id == entity_id], provider='zvt')
        else:
            del_data(FactorState, filters=[FactorState.factor_name == self.factor_name], provider='zvt')
            del_data(self.factor_schema, provider='zvt')

    def decode_state(self, state: str):
        return json.loads(state, object_hook=self.factor_state_object_hook())

    def encode_state(self, state: object):
        return json.dumps(state, cls=self.factor_encoder())

    def factor_encoder(self):
        return None

    def pre_compute(self):
        if not self.only_load_factor and not pd_is_not_null(self.pipe_df):
            self.pipe_df = self.data_df

    def do_compute(self):
        self.logger.info('compute factor start')
        self.compute_factor()
        self.logger.info('compute factor finish')

        self.logger.info('compute result start')
        self.compute_result()
        self.logger.info('compute result finish')

    def compute_factor(self):
        if self.only_load_factor:
            return
            # 无状态的转换运算
        if pd_is_not_null(self.data_df) and self.transformer:
            self.pipe_df = self.transformer.transform(self.data_df)
        else:
            self.pipe_df = self.data_df

        # 有状态的累加运算
        if pd_is_not_null(self.pipe_df) and self.accumulator:
            self.factor_df, self.states = self.accumulator.acc(self.pipe_df, self.factor_df, self.states)
        else:
            self.factor_df = self.pipe_df

    def compute_result(self):
        pass

    def after_compute(self):
        if self.only_load_factor:
            return
        if self.keep_all_timestamp:
            self.fill_gap()

        if self.need_persist and pd_is_not_null(self.factor_df):
            self.persist_factor()

    def compute(self):
        self.pre_compute()

        self.logger.info(f'[[[ ~~~~~~~~factor:{self.factor_name} ~~~~~~~~]]]')
        self.logger.info('do_compute start')
        start_time = time.time()
        self.do_compute()
        cost_time = time.time() - start_time
        self.logger.info('do_compute finished,cost_time:{}s'.format(cost_time))

        self.logger.info('after_compute start')
        start_time = time.time()
        self.after_compute()
        cost_time = time.time() - start_time
        self.logger.info('after_compute finished,cost_time:{}s'.format(cost_time))
        self.logger.info(f'[[[ ^^^^^^^^factor:{self.factor_name} ^^^^^^^^]]]')

    def drawer_main_df(self) -> Optional[pd.DataFrame]:
        if self.only_load_factor:
            return self.factor_df
        return self.data_df

    def drawer_factor_df_list(self) -> Optional[List[pd.DataFrame]]:
        if (self.transformer is not None or self.accumulator is not None) and pd_is_not_null(self.factor_df):
            return [self.factor_df]
        return None

    def drawer_sub_df_list(self) -> Optional[List[pd.DataFrame]]:
        if (self.transformer is not None or self.accumulator is not None) and pd_is_not_null(self.result_df):
            return [self.result_df]
        return None

    def drawer_annotation_df(self) -> Optional[pd.DataFrame]:
        def order_type_flag(order_type):
            if order_type:
                return 'B'

        if pd_is_not_null(self.result_df):
            annotation_df = self.result_df.copy()
            annotation_df = annotation_df[annotation_df['score']]
            annotation_df['value'] = self.factor_df.loc[annotation_df.index]['close']
            annotation_df['flag'] = annotation_df['score'].apply(lambda x: order_type_flag(x))
            return annotation_df

    def fill_gap(self):
        # 该操作较慢，只适合做基本面的运算
        idx = pd.date_range(self.start_timestamp, self.end_timestamp)
        new_index = pd.MultiIndex.from_product([self.result_df.index.levels[0], idx],
                                               names=[self.category_field, self.time_field])
        self.result_df = self.result_df.loc[~self.result_df.index.duplicated(keep='first')]
        self.result_df = self.result_df.reindex(new_index)
        self.result_df = self.result_df.groupby(level=0).fillna(method=self.fill_method, limit=self.effective_number)

    def on_data_loaded(self, data: pd.DataFrame):
        self.compute()

    def on_data_changed(self, data: pd.DataFrame):
        """
        overwrite it for computing after data added

        Parameters
        ----------
        data :
        """
        self.compute()

    def on_entity_data_changed(self, entity, added_data: pd.DataFrame):
        """
        overwrite it for computing after entity data added

        Parameters
        ----------
        entity :
        added_data :
        """
        pass

    def persist_factor(self):
        df = self.factor_df.copy()
        # encode json columns
        if pd_is_not_null(df) and self.factor_col_map_object_hook():
            for col in self.factor_col_map_object_hook():
                if col in df.columns:
                    df[col] = df[col].apply(lambda x: json.dumps(x, cls=self.factor_encoder()))

        if self.states:
            session = get_db_session(provider='zvt', data_schema=FactorState)
            g = df.groupby(level=0)

            for entity_id in self.states:
                state = self.states[entity_id]
                try:
                    if state:
                        domain_id = f'{self.factor_name}_{entity_id}'
                        factor_state: FactorState = session.query(FactorState).get(domain_id)
                        state_str = self.encode_state(state)
                        if factor_state:
                            factor_state.state = state_str
                        else:
                            factor_state = FactorState(id=domain_id, entity_id=entity_id,
                                                       factor_name=self.factor_name,
                                                       state=state_str)
                        session.add(factor_state)
                        session.commit()
                    if entity_id in g.groups:
                        df_to_db(df=df.loc[(entity_id,)], data_schema=self.factor_schema, provider='zvt',
                                 force_update=False)
                except Exception as e:
                    self.logger.error(f'{self.factor_name} {entity_id} save state error')
                    self.logger.exception(e)
                    # clear them if error happen
                    self.clear_state_data(entity_id)
        else:
            df_to_db(df=df, data_schema=self.factor_schema, provider='zvt', force_update=False)


class FilterFactor(Factor):
    factor_type = FactorType.filter


class ScoreFactor(Factor):
    factor_type = FactorType.score
    scorer: Scorer = None

    def compute_result(self):
        super().compute_result()
        if pd_is_not_null(self.factor_df) and self.scorer:
            self.result_df = self.scorer.score(self.factor_df)


# the __all__ is generated
__all__ = ['Indicator', 'Transformer', 'Accumulator', 'Scorer', 'FactorType', 'register_class', 'FactorMeta',
           'FactorState', 'Factor', 'FilterFactor', 'ScoreFactor']



# -*- coding: utf-8 -*-

import dash_core_components as dcc

from zvtm.api.kdata import get_kdata_schema
from zvtm.api.trader_info_api import OrderReader, AccountStatsReader
from zvtm.contract.api import decode_entity_id
from zvtm.contract.drawer import Drawer
from zvtm.contract.reader import DataReader
from zvtm.contract.zvt_context import tradable_schema_map
from zvtm.utils.pd_utils import pd_is_not_null


def order_type_color(order_type):
    if order_type == 'order_long' or order_type == 'order_close_short':
        return "#ec0000"
    else:
        return "#00da3c"


def order_type_flag(order_type):
    if order_type == 'order_long' or order_type == 'order_close_short':
        return 'B'
    else:
        return 'S'


def get_trading_signals_figure(order_reader: OrderReader,
                               entity_id: str,
                               start_timestamp=None,
                               end_timestamp=None,
                               adjust_type=None):
    entity_type, _, _ = decode_entity_id(entity_id)

    data_schema = get_kdata_schema(entity_type=entity_type, level=order_reader.level, adjust_type=adjust_type)
    if not start_timestamp:
        start_timestamp = order_reader.start_timestamp
    if not end_timestamp:
        end_timestamp = order_reader.end_timestamp
    kdata_reader = DataReader(entity_ids=[entity_id], data_schema=data_schema,
                              entity_schema=tradable_schema_map.get(entity_type),
                              start_timestamp=start_timestamp,
                              end_timestamp=end_timestamp,
                              level=order_reader.level)

    # generate the annotation df
    order_reader.move_on(timeout=0)
    df = order_reader.data_df.copy()
    df = df[df.entity_id == entity_id].copy()
    if pd_is_not_null(df):
        df['value'] = df['order_price']
        df['flag'] = df['order_type'].apply(lambda x: order_type_flag(x))
        df['color'] = df['order_type'].apply(lambda x: order_type_color(x))
    print(df.tail())

    drawer = Drawer(main_df=kdata_reader.data_df, annotation_df=df)
    return drawer.draw_kline(show=False, height=800)


def get_account_stats_figure(account_stats_reader: AccountStatsReader):
    graph_list = []

    # 账户统计曲线
    if account_stats_reader:
        fig = account_stats_reader.draw_line(show=False)

        for trader_name in account_stats_reader.trader_names:
            graph_list.append(dcc.Graph(
                id='{}-account'.format(trader_name),
                figure=fig))

    return graph_list



# -*- coding: utf-8 -*-
from sqlalchemy import String, Column, Float

from zvtm.contract import Mixin


class KdataCommon(Mixin):
    provider = Column(String(length=32))
    code = Column(String(length=32))
    name = Column(String(length=32))
    # Enum constraint is not extendable
    # level = Column(Enum(IntervalLevel, values_callable=enum_value))
    level = Column(String(length=32))

    # 开盘价
    open = Column(Float)
    # 收盘价
    close = Column(Float)
    # 最高价
    high = Column(Float)
    # 最低价
    low = Column(Float)
    # 成交量
    volume = Column(Float)
    # 成交金额
    turnover = Column(Float)
    # 涨跌幅
    change_pct = Column(Float)
    # 换手率
    turnover_rate = Column(Float)


class TickCommon(Mixin):
    provider = Column(String(length=32))
    code = Column(String(length=32))
    name = Column(String(length=32))
    level = Column(String(length=32))

    order = Column(String(length=32))
    price = Column(Float)
    volume = Column(Float)
    turnover = Column(Float)
    direction = Column(String(length=32))
    order_type = Column(String(length=32))


class BlockKdataCommon(KdataCommon):
    pass


class IndexKdataCommon(KdataCommon):
    pass


class EtfKdataCommon(KdataCommon):
    turnover_rate = Column(Float)

    # ETF 累计净值（货币 ETF 为七日年化)
    cumulative_net_value = Column(Float)

class StockKdataCommon(KdataCommon):
    pass


class StockusKdataCommon(KdataCommon):
    pass


class StockhkKdataCommon(KdataCommon):
    pass


# the __all__ is generated
__all__ = ['KdataCommon', 'TickCommon', 'BlockKdataCommon', 'IndexKdataCommon', 'EtfKdataCommon', 'StockKdataCommon', 'StockusKdataCommon', 'StockhkKdataCommon']

# __init__.py structure:
# common code of the package
# export interface in __all__ which contains __all__ of its sub modules

# import all from submodule trade_day
from .trade_day import *
from .trade_day import __all__ as _trade_day_all
__all__ += _trade_day_all

# import all from submodule stockhk
from .stockhk import *
from .stockhk import __all__ as _stockhk_all
__all__ += _stockhk_all

# import all from submodule stockus
from .stockus import *
from .stockus import __all__ as _stockus_all
__all__ += _stockus_all

# import all from submodule index
from .index import *
from .index import __all__ as _index_all
__all__ += _index_all

# import all from submodule etf
from .etf import *
from .etf import __all__ as _etf_all
__all__ += _etf_all

# import all from submodule stock
from .stock import *
from .stock import __all__ as _stock_all
__all__ += _stock_all

# import all from submodule block
from .block import *
from .block import __all__ as _block_all
__all__ += _block_all


# -*- coding: utf-8 -*-
# the __all__ is generated
__all__ = []

# __init__.py structure:
# common code of the package
# export interface in __all__ which contains __all__ of its sub modules

# import all from submodule misc
from .misc import *
from .misc import __all__ as _misc_all
__all__ += _misc_all

# import all from submodule quotes
from .quotes import *
from .quotes import __all__ as _quotes_all
__all__ += _quotes_all

# import all from submodule meta
from .meta import *
from .meta import __all__ as _meta_all
__all__ += _meta_all

# import all from submodule fundamental
from .fundamental import *
from .fundamental import __all__ as _fundamental_all
__all__ += _fundamental_all

# import all from submodule common
from .common import *
from .common import __all__ as _common_all
__all__ += _common_all

# import all from submodule overall
from .overall import *
from .overall import __all__ as _overall_all
__all__ += _overall_all


# -*- coding: utf-8 -*-

from sqlalchemy import Column, String, Float
from sqlalchemy.orm import declarative_base

from zvtm.contract import Portfolio, PortfolioStockHistory
from zvtm.contract.register import register_schema, register_entity

IndexMetaBase = declarative_base()


# 指数
@register_entity(entity_type='index')
class Index(IndexMetaBase, Portfolio):
    __tablename__ = 'indexs'

    # 发布商
    publisher = Column(String(length=64))
    # 类别
    # see IndexCategory
    category = Column(String(length=64))
    # 基准点数
    base_point = Column(Float)


class IndexStock(IndexMetaBase, PortfolioStockHistory):
    __tablename__ = 'index_stock'


register_schema(providers=['exchange','joinquant'], db_name='index_meta', schema_base=IndexMetaBase)
# the __all__ is generated
__all__ = ['Index', 'IndexStock']


# -*- coding: utf-8 -*-
from typing import List, Optional

import numpy as np
import pandas as pd

from zvtm.factors.algorithm import MacdTransformer
from zvtm.factors.technical_factor import TechnicalFactor


class MacdFactor(TechnicalFactor):
    transformer = MacdTransformer(count_live_dead=True)

    def drawer_factor_df_list(self) -> Optional[List[pd.DataFrame]]:
        return None

    def drawer_sub_df_list(self) -> Optional[List[pd.DataFrame]]:
        return [self.factor_df[['diff', 'dea', 'macd']]]

    def drawer_sub_col_chart(self) -> Optional[dict]:
        return {'diff': 'line',
                'dea': 'line',
                'macd': 'bar'}


class BullFactor(MacdFactor):
    def compute_result(self):
        super().compute_result()
        self.result_df = self.factor_df['bull'].to_frame(name='score')


class KeepBullFactor(BullFactor):
    keep_window = 20

    def compute_result(self):
        super().compute_result()
        df = self.result_df['score'].groupby(level=0).rolling(window=self.keep_window,
                                                              min_periods=self.keep_window).apply(
            lambda x: np.logical_and.reduce(x))
        df = df.reset_index(level=0, drop=True)
        self.result_df['score'] = df


# 金叉 死叉 持续时间 切换点
class LiveOrDeadFactor(MacdFactor):
    pattern = [-5, 1]

    def compute_result(self):
        super().compute_result()
        self.factor_df['pre'] = self.factor_df['live_count'].shift()
        s = (self.factor_df['pre'] <= self.pattern[0]) & (self.factor_df['live_count'] >= self.pattern[1])
        self.result_df = s.to_frame(name='score')


class GoldCrossFactor(MacdFactor):
    def compute_result(self):
        super().compute_result()
        s = self.factor_df['live'] == 1
        self.result_df = s.to_frame(name='score')


# the __all__ is generated
__all__ = ['MacdFactor', 'BullFactor', 'KeepBullFactor', 'LiveOrDeadFactor', 'GoldCrossFactor']


# -*- coding: utf-8 -*-
from zvtm.utils.utils import chrome_copy_header_to_dict

SSE_KDATA_HEADER = chrome_copy_header_to_dict('''
Host: yunhq.sse.com.cn:32041
Connection: keep-alive
Pragma: no-cache
Cache-Control: no-cache
User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36
Accept: */*
Referer: http://www.sse.com.cn/market/price/trends/
Accept-Encoding: gzip, deflate
Accept-Language: zh-CN,zh;q=0.8,en;q=0.6
Cookie: yfx_c_g_u_id_10000042=_ck17072000172016360411059933357; yfx_f_l_v_t_10000042=f_t_1500481040618__r_t_1507560823182__v_t_1507561607501__r_c_1; VISITED_MENU=%5B%228451%22%2C%228453%22%5D
''')

DEFAULT_SH_HEADER = chrome_copy_header_to_dict('''
Accept: */*
Accept-Encoding: gzip, deflate
Accept-Language: zh-CN,zh;q=0.9,en;q=0.8
Connection: keep-alive
Cookie: yfx_c_g_u_id_10000042=_ck21052517524516575117884257529; yfx_f_l_v_t_10000042=f_t_1621936365643__r_t_1621936365643__v_t_1621936365643__r_c_0; VISITED_MENU=%5B%228466%22%2C%228528%22%5D; JSESSIONID=3C970EA11443D2B7F3844558174C2CDB
Host: query.sse.com.cn
Referer: http://www.sse.com.cn/
User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Safari/537.36
''')

DEFAULT_SZ_HEADER = chrome_copy_header_to_dict('''
Accept:text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8
Accept-Encoding:gzip, deflate, sdch
Accept-Language:zh-CN,zh;q=0.8,en;q=0.6
Connection:keep-alive
Host:www.szse.cn
Referer:http://www.szse.cn/main/marketdata/jypz/colist/
Upgrade-Insecure-Requests:1
User-Agent:Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2490.76 Mobile Safari/537.36
''')

DEFAULT_TICK_HEADER = chrome_copy_header_to_dict('''
Host: market.finance.sina.com.cn
Accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8
Accept-Encoding: gzip, deflate, sdch
Accept-Language: zh-CN,zh;q=0.8,en;q=0.6
Cookie: U_TRS1=000000b3.fffc2f53.566f784b.2900c0a6; UOR=www.baidu.com,blog.sina.com.cn,; SINAGLOBAL=182.139.185.185_1450145868.176138; vjuids=-fde6fe2f2.151a36dec26.0.113c0d8; SGUID=1450145869010_36029656; Apache=182.139.185.132_1457087046.693218; ULV=1457087047846:3:2:2:182.139.185.132_1457087046.693218:1457087047557; U_TRS2=0000009f.95a9161b.57038c9e.11b3a69f; SessionID=e2ajs2t3k4emag633adfjho861; SINA_FINANCE_SELECT_TYPE=stock; SCF=AhFAeZgjMw_ozKbe2MXZtX2fG8ZrLemvX24bd57wciL98hfpOY_4hN8aYqvvaxZDbINdQq3XqZSrS-ubJo-5VUI.; sso_info=v02m6alo5qztbaYloWum6akpp2WpaSPk4S1jpOYsYyjlLONg5DA; SUP=cv%3D1%26bt%3D1470117772%26et%3D1470204172%26d%3D40c3%26i%3De2a8%26us%3D0%26vf%3D0%26vt%3D0%26ac%3D2%26st%3D0%26lt%3D7%26uid%3D1596125344%26user%3Djackofxuan%2540gmail.com%26ag%3D4%26name%3Djackofxuan%2540gmail.com%26nick%3Dvaanni%26sex%3D%26ps%3D0%26email%3D%26dob%3D%26ln%3D%26os%3D%26fmp%3D%26lcp%3D2011-12-26%252017%253A00%253A23; __utma=269849203.900451459.1468839849.1468839849.1471431141.2; __utmc=269849203; __utmz=269849203.1471431141.2.2.utmcsr=finance.sina.com.cn|utmccn=(referral)|utmcmd=referral|utmcct=/; SINABLOGNUINFO=1596125344.5f22f0a0.vannii; SINA_FINANCE=%3A1596125344%3A4; hk_visited_stocks=02338; WEB2_APACHE2_GD=1401fbd37fe42dfbc571af9dacb5e317; _s_upa=25; lxlrtst=1475225080_o; SR_SEL=1_511; SINA_PORTFOLIO=sh601600%2Csh600089%2Csz000528%2Csh600150%2Csz000338%2Csh600031%2Csh600199%2Csh000001%2Csz399001%2Csz000778%2Csh600315%2Csh600570%2Csh601608%2Csz000099%2Csh600439%2Csz000521%2Csz300276%2Csz000783%2Csh600396%2Csz300336%2Csz000751%2Csz000848%2Csh600711%2Csh600880%2Csh600362%2Csh600060%2Csz000680%2Csz002051; ArtiFSize=14; visited_uss=gb_goog%7Cgb_sina%7Cgb_.dji; FINA_DMHQ=0; hqEtagMode=1; VISITED_FUTURE=hf_CL_0%2Chf_GC_0%2Chf_SI_0%2Chf_CAD_0%2CCF1609_1%2CRB0_1%2Chf_DJS_0%2CCFF_RE_IF1606_2%2CCFF_RE_IF1510_2%2CCFF_RE_IC1606_2%2CRM0_1%2CRB1610_1; lxlrttp=1476092678; FINA_V5_HQ=0; SUB=_2A251BPWoDeTxGedL4lQQ8ivPzziIHXVWcGBgrDV_PUNbm9BeLXLzkW8iLp81bNoV5si_9RFc9X4XY1wBcg..; SUBP=0033WrSXqPxfM725Ws9jqgMF55529P9D9W5J7vXaOMNYJQ0d27EZPWIF5NHD95QpSK.ceKzfe0BXWs4Dqcj.i--fi-zRiKnEi--fiKLhi-iWi--Xi-isi-88i--Ri-2piKyh; ALF=1507965304; FIN_ALL_VISITED=sz000338%2Csh000001%2Csz000680%2Csz399006%2Csh600028%2Csh600315%2Csh600031%2Csh601608%2Csh601038%2Csh601628%2CCL%2Csh600150%2Csz300369%2Csz300079%2CAGTD%2Csz300336%2Csz300276%2Cgoog%2Csina%2CGC%2CSI%2Csh600004%2Csh600000%2Csz300374%2Csh600199%2Csh600439%2Csz000783%2Csz000912%2CCAD%2Csz002628%2Csh600362%2Csz002051%2Csz000930%2Csz000528%2Csh603568%2Csh600060%2Csh600635%2Csz150266%2Csz000651%2Csz000099%2Csh600570%2Csz000002%2Csh600048%2Csz000839%2Csz002223%2Csh600880%2Csz000521%2Csz000878%2Csz399001%2Csh600396%2Csz000751%2Csh600016%2Csz000915%2Csh600779%2Csh600497%2Csh601600%2Csh600111%2Csh600550%2Csz000426%2Csz002237; FINA_LV2_S_2=1596125344; FINA_LV2_S_2_B=0; rotatecount=1; vjlast=1476433318; FINA_V_S_2=sz000338,sh000001,sz000680,sz399006,sh600028,sh600315,sh600031,sh601608,sh601038,sh601628,sh600150,sz300369,sz300079,sz300336,sz300276,sh600004,sh600000,sz300374,sh600199,sh600439
Referer:market.finance.sina.com.cn
Upgrade-Insecure-Requests: 1
User-Agent: Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.82 Safari/537.36
''')

DEFAULT_KDATA_HEADER = chrome_copy_header_to_dict('''
Accept:text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8
Accept-Encoding:gzip, deflate, sdch
Accept-Language:zh-CN,zh;q=0.8,en;q=0.6
Cache-Control:max-age=0
Connection:keep-alive
Host:vip.stock.finance.sina.com.cn
Referer:vip.stock.finance.sina.com.cn
Upgrade-Insecure-Requests:1
User-Agent:Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2490.76 Mobile Safari/537.36
''')

TONGHUASHUN_GN_HEADER = chrome_copy_header_to_dict('''
Accept:text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8
Accept-Encoding:gzip, deflate, sdch
Accept-Language:zh-CN,zh;q=0.8,en;q=0.6
Cache-Control:max-age=0
Connection:keep-alive
Cookie:Hm_lvt_ab89213e83c551bf095446c08bf64988=1477116434; Hm_lpvt_ab89213e83c551bf095446c08bf64988=1477116434; historystock=600126; spversion=20130314; Hm_lvt_78c58f01938e4d85eaf619eae71b4ed1=1477105757,1477116434; Hm_lpvt_78c58f01938e4d85eaf619eae71b4ed1=1477362626
Host:q.10jqka.com.cn
Upgrade-Insecure-Requests:1
User-Agent:Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2490.76 Mobile Safari/537.36
''')

TONGHUASHUN_KDATA_HEADER = chrome_copy_header_to_dict('''
Accept-Encoding: gzip, deflate
Accept-Language: zh-CN,zh;q=0.8,en;q=0.6
User-Agent: Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36
Accept: */*
Referer: http://stockpage.10jqka.com.cn/HQ_v4.html
Cookie: __utma=156575163.1843700306.1488352720.1499234323.1502172029.4; __utmc=156575163; __utmz=156575163.1488352720.1.1.utmcsr=(direct)|utmccn=(direct)|utmcmd=(none); spversion=20130314; Hm_lvt_78c58f01938e4d85eaf619eae71b4ed1=1507300869; Hm_lpvt_78c58f01938e4d85eaf619eae71b4ed1=1508464591; historystock=603189%7C*%7C300295%7C*%7C600839%7C*%7C000338%7C*%7C002194; log=; v=AREjaxfGLPcJoUDd5wHRp1QiKRaufoSAL_MpD_OlDDL35T_CO86VwL9CPa2D
Connection: keep-alive
''')

DEFAULT_BALANCE_SHEET_HEADER = chrome_copy_header_to_dict('''
Host: money.finance.sina.com.cn
User-Agent: Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:54.0) Gecko/20100101 Firefox/54.0
Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8
Accept-Language: en-US,en;q=0.5
Accept-Encoding: gzip, deflate
Referer: http://vip.stock.finance.sina.com.cn/corp/go.php/vFD_CashFlow/stockid/000338/ctrl/part/displaytype/4.phtml
Cookie: U_TRS1=000000be.c95848c3.59817e10.a54886e2; U_TRS2=000000be.c96a48c3.59817e10.a91795e2; UOR=,vip.stock.finance.sina.com.cn,; ULV=1501658645426:2:2:2:182.148.114.190_1501658642.469995:1501658642409; SINAGLOBAL=182.148.114.190_1501658642.469991; Apache=182.148.114.190_1501658642.469995; _s_upa=1; SUB=_2A250hQ5nDeRhGedL4lQQ8ivPzziIHXVX83ivrDV_PUNbm9BeLXigkW8-niaOks2yNkw8lYo-TvoqGk6nRA..; SUBP=0033WrSXqPxfM725Ws9jqgMF55529P9D9W5J7vXaOMNYJQ0d27EZPWIF5NHD95QpSK.ceKzfe0BXWs4Dqcj.i--fi-zRiKnEi--fiKLhi-iWi--Xi-isi-88i--Ri-2piKyh; SCF=At4whqZZyjTBTvcLfR0tyqIpfHUX2VOK-qvBVHkbyahiCVcr4-8NjJQGHwCaTtkQJ0SPmrzvZARwtEkL1I_46z8.; ALF=1533194679; sso_info=v02m6alo5qztbaYloWum6akpp2WpaSPk4S1jpOYsYyjlLONg5DA; FINANCE2=f7634b1d12920e2763ffc0dc463ef6bb
Connection: keep-alive
Upgrade-Insecure-Requests: 1
''')

DEFAULT_SH_SUMMARY_HEADER = chrome_copy_header_to_dict('''
Host: query.sse.com.cn
Connection: keep-alive
User-Agent: Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36
Accept: */*
Referer: http://www.sse.com.cn/market/stockdata/overview/day/
Accept-Encoding: gzip, deflate
Accept-Language: zh-CN,zh;q=0.8,en;q=0.6
Cookie: yfx_c_g_u_id_10000042=_ck17122009304714819234313401740; VISITED_COMPANY_CODE=%5B%22000016%22%5D; VISITED_INDEX_CODE=%5B%22000016%22%5D; yfx_f_l_v_t_10000042=f_t_1513733447386__r_t_1515716891222__v_t_1515721033042__r_c_3; VISITED_MENU=%5B%228464%22%2C%229666%22%2C%229668%22%2C%229669%22%2C%228454%22%2C%228460%22%2C%229665%22%2C%228459%22%2C%229692%22%2C%228451%22%2C%228466%22%5D
''')

DEFAULT_SH_ETF_LIST_HEADER = chrome_copy_header_to_dict('''
Host: query.sse.com.cn
User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36
Accept: */*
Referer: http://www.sse.com.cn/assortment/fund/etf/list/
Accept-Encoding: gzip, deflate
Accept-Language: zh-CN,zh;q=0.9
Cookie: yfx_c_g_u_id_10000042=_ck19062609443812815766114343798; VISITED_COMPANY_CODE=%5B%22510300%22%5D; VISITED_FUND_CODE=%5B%22510300%22%5D; VISITED_MENU=%5B%228307%22%2C%228823%22%2C%228547%22%2C%228556%22%2C%228549%22%2C%2210848%22%2C%228550%22%5D; yfx_f_l_v_t_10000042=f_t_1561513478278__r_t_1561692626758__v_t_1561695738302__r_c_1
Connection: keep-alive
''')

EASTMONEY_ETF_NET_VALUE_HEADER = chrome_copy_header_to_dict('''
User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36
Referer: http://fund.eastmoney.com/
''')

DEFAULT_HEADER = chrome_copy_header_to_dict('''
User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36
''')
# the __all__ is generated
__all__ = []


# -*- coding: utf-8 -*-
# this file is generated by gen_kdata_schema function, dont't change it
from sqlalchemy.orm import declarative_base

from zvtm.contract.register import register_schema
from zvtm.domain.quotes import BlockKdataCommon

KdataBase = declarative_base()


class Block1monKdata(KdataBase, BlockKdataCommon):
    __tablename__ = 'block_1mon_kdata'


register_schema(providers=['em'], db_name='block_1mon_kdata', schema_base=KdataBase, entity_type='block')

# the __all__ is generated
__all__ = ['Block1monKdata']


# -*- coding: utf-8 -*-
import logging
from enum import Enum

from zvtm.api import get_recent_report
from zvtm.contract import ActorType
from zvtm.domain import StockActorSummary
from zvtm.tag.dataset.stock_tags import StockTags
from zvtm.tag.tag import StockTagger
from zvtm.utils import pd_is_not_null

logger = logging.getLogger(__name__)


class ActorTag(Enum):
    # 基金心头好
    fund_love = 'fund_love'
    # 基金看不起
    fund_not_care = 'fund_not_care'


class ActorTagger(StockTagger):
    def tag(self, timestamp):
        df = get_recent_report(data_schema=StockActorSummary, timestamp=timestamp,
                               filters=[StockActorSummary.actor_type == ActorType.raised_fund.value])
        if not pd_is_not_null(df):
            logger.error(f'no StockActorSummary data at {timestamp}')
            return

        df = df.set_index('entity_id')

        fund_love_ids = df[df['holding_ratio'] >= 0.05].index.tolist()
        fund_not_care_ids = df[df['holding_ratio'] < 0.05].index.tolist()

        fund_love_domains = self.get_tag_domains(entity_ids=fund_love_ids, timestamp=timestamp,
                                                 actor_tag=ActorTag.fund_love.value)
        fund_not_care_domains = self.get_tag_domains(entity_ids=fund_not_care_ids, timestamp=timestamp,
                                                     actor_tag=ActorTag.fund_not_care.value)
        self.session.add_all(fund_love_domains)
        self.session.add_all(fund_not_care_domains)
        self.session.commit()


if __name__ == '__main__':
    # ActorTagger().run()
    print(StockTags.query_data(start_timestamp='2021-08-31', filters=[StockTags.actor_tag != None]))



import dash_bootstrap_components as dbc
import dash_html_components as html
from dash.dependencies import Input, Output

from zvtm import init_plugins
from zvtm.ui import zvt_app
from zvtm.ui.apps import factor_app


def serve_layout():
    layout = html.Div(
        children=[
            # banner
            html.Div(
                className="zvt-banner",
                children=html.H2(className="h2-title", children="ZVT")
            ),
            dbc.CardHeader(
                dbc.Tabs(
                    [
                        dbc.Tab(label="factor", tab_id="tab-factor", label_style={}, tab_style={"width": "100px"})
                    ],
                    id="card-tabs",
                    card=True,
                    active_tab="tab-factor",
                )
            ),
            dbc.CardBody(html.P(id="card-content", className="card-text")),
        ]
    )

    return layout


@zvt_app.callback(
    Output("card-content", "children"), [Input("card-tabs", "active_tab")]
)
def tab_content(active_tab):
    if 'tab-factor' == active_tab:
        return factor_app.factor_layout()


zvt_app.layout = serve_layout


def main():
    init_plugins()
    zvt_app.run_server(debug=True)
    # zvt_app.run_server()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import requests

from zvtm.contract.api import get_entities
from zvtm.contract.recorder import Recorder
from zvtm.domain.meta.stock_meta import StockDetail, Stock
from zvtm.recorders.exchange.exchange_stock_meta_recorder import ExchangeStockMetaRecorder
from zvtm.utils.time_utils import to_pd_timestamp
from zvtm.utils.utils import to_float, pct_to_float


class EastmoneyStockRecorder(ExchangeStockMetaRecorder):
    data_schema = Stock
    provider = 'eastmoney'


class EastmoneyStockDetailRecorder(Recorder):
    provider = 'eastmoney'
    data_schema = StockDetail

    def __init__(self, force_update=False, sleeping_time=5, code=None, codes=None) -> None:
        super().__init__(force_update, sleeping_time)

        # get list at first
        EastmoneyStockRecorder().run()

        if codes is None and code is not None:
            self.codes = [code]
        else:
            self.codes = codes
        filters = None
        if not self.force_update:
            filters = [StockDetail.profile.is_(None)]
        self.entities = get_entities(session=self.session,
                                         entity_schema=StockDetail,
                                         exchanges=None,
                                         codes=self.codes,
                                         filters=filters,
                                         return_type='domain',
                                         provider=self.provider)

    def run(self):
        for security_item in self.entities:
            assert isinstance(security_item, StockDetail)

            if security_item.exchange == 'sh':
                fc = "{}01".format(security_item.code)
            if security_item.exchange == 'sz':
                fc = "{}02".format(security_item.code)

            # 基本资料
            param = {"color": "w", "fc": fc, "SecurityCode": "SZ300059"}
            resp = requests.post('https://emh5.eastmoney.com/api/GongSiGaiKuang/GetJiBenZiLiao', json=param)
            resp.encoding = 'utf8'

            resp_json = resp.json()['Result']['JiBenZiLiao']

            security_item.profile = resp_json['CompRofile']
            security_item.main_business = resp_json['MainBusiness']
            security_item.date_of_establishment = to_pd_timestamp(resp_json['FoundDate'])

            # 关联行业
            industries = ','.join(resp_json['Industry'].split('-'))
            security_item.industries = industries

            # 关联概念
            security_item.concept_indices = resp_json['Block']

            # 关联地区
            security_item.area_indices = resp_json['Provice']

            self.sleep()

            # 发行相关
            param = {"color": "w", "fc": fc}
            resp = requests.post('https://emh5.eastmoney.com/api/GongSiGaiKuang/GetFaXingXiangGuan', json=param)
            resp.encoding = 'utf8'

            resp_json = resp.json()['Result']['FaXingXiangGuan']

            security_item.issue_pe = to_float(resp_json['PEIssued'])
            security_item.price = to_float(resp_json['IssuePrice'])
            security_item.issues = to_float(resp_json['ShareIssued'])
            security_item.raising_fund = to_float((resp_json['NetCollection']))
            security_item.net_winning_rate = pct_to_float(resp_json['LotRateOn'])

            self.session.commit()

            self.logger.info('finish recording stock meta for:{}'.format(security_item.code))

            self.sleep()


if __name__ == '__main__':
    # init_log('china_stock_meta.log')

    recorder = EastmoneyStockRecorder()
    recorder.run()
    StockDetail.record_data(codes=['000338', '000777'], provider='eastmoney')
# the __all__ is generated
__all__ = ['EastmoneyStockRecorder', 'EastmoneyStockDetailRecorder']